#!/usr/bin/env python3
"""Implements the automated A31 checklist fixing workflow."""

from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from noveler.domain.interfaces.console_service_protocol import IConsoleService
    from noveler.domain.interfaces.path_service_protocol import IPathService

from noveler.application.base.abstract_use_case import AbstractUseCase
from noveler.domain.entities.a31_checklist_item import A31ChecklistItem
from noveler.domain.entities.auto_fix_session import AutoFixSession
from noveler.domain.services.a31_auto_fix_service import A31AutoFixService
from noveler.domain.services.a31_evaluation_service import A31EvaluationService
from noveler.domain.value_objects.a31_evaluation_result import EvaluationResult
from noveler.domain.value_objects.a31_fix_level import FixLevel
from noveler.domain.value_objects.a31_session_id import SessionId


class A31AutoFixUseCase(AbstractUseCase[object, AutoFixSession]):
    """Run the automated A31 checklist fix workflow for an episode."""

    def __init__(
        self,
        logger_service=None,
        unit_of_work=None,
        *,
        console_service: IConsoleService | None = None,
        path_service: IPathService | None = None,
        episode_repository=None,
        project_repository=None,
        evaluation_service: A31EvaluationService | None = None,
        auto_fix_service: A31AutoFixService | None = None,
        a31_checklist_repository=None,
        **legacy_kwargs,
    ) -> None:
        """Initialise the use case with required infrastructure services.

        Args:
            logger_service (ILoggerService): Logger used for diagnostics.
            unit_of_work (IUnitOfWork): Unit of Work coordinating repositories.
            console_service (IConsoleService, optional): Console writer used for
                status output. Defaults to ``None``.
            path_service (IPathService, optional): Path helper for project
                lookups. Defaults to ``None``.
        """
        # 基底クラス初期化（共通サービス）
        super().__init__(
            logger_service=logger_service,
            unit_of_work=unit_of_work,
            console_service=console_service,
            path_service=path_service,
            **legacy_kwargs,
        )

        if self.unit_of_work is None:
            if episode_repository is None or project_repository is None:
                msg = "unit_of_work または episode_repository/project_repository を指定してください"
                raise ValueError(msg)

            from noveler.infrastructure.unit_of_work import UnitOfWork

            unit_of_work = UnitOfWork(
                episode_repository=episode_repository,
                project_repository=project_repository,
            )
        else:
            unit_of_work = self.unit_of_work

        self._unit_of_work = unit_of_work
        self._episode_repository = episode_repository or getattr(unit_of_work, "episode_repository", None)
        self._project_repository = project_repository or getattr(unit_of_work, "project_repository", None)

        if self._episode_repository is None or self._project_repository is None:
            msg = "A31AutoFixUseCase には episode_repository と project_repository が必要です"
            raise ValueError(msg)

        self._logger_service = self.logger_service
        self._console_service = self.console_service
        self._evaluation_service = evaluation_service
        self._auto_fix_service = auto_fix_service
        self._checklist_repository = a31_checklist_repository

    def execute(
        self, project_name: str, episode_number: int, fix_level: FixLevel, items_to_fix: list[str]
    ) -> AutoFixSession:
        """Execute the automatic fixing workflow for the provided episode."""
        self._logger_service.info(f"A31自動修正開始: {project_name} #{episode_number}")

        with self._unit_of_work.transaction():
            try:
                episode_content = self._unit_of_work.episode_repository.get_episode_content(
                    project_name, episode_number
                )

                checklist_items = self._project_repository.get_checklist_items(project_name, items_to_fix)
                if not checklist_items:
                    msg = "指定されたチェックリスト項目が見つかりません"
                    raise ValueError(msg)

                session_id = SessionId.generate()
                target_file = self._get_episode_file_path(project_name, episode_number)
                session = AutoFixSession(
                    session_id=session_id, target_file=target_file, fix_level=fix_level, items_to_fix=items_to_fix
                )

                evaluation_service = self._create_evaluation_service()
                auto_fix_service = self._create_auto_fix_service()

                evaluation_results = evaluation_service.evaluate_all_items(
                    checklist_items, episode_content, self._get_project_metadata(project_name)
                )

                fixed_content, fix_results = auto_fix_service.apply_fixes(
                    episode_content, evaluation_results, checklist_items, fix_level
                )

                for fix_result in fix_results:
                    session.add_result(fix_result)

                if fixed_content != episode_content:
                    self.save_fixed_content(session, fixed_content)

                if self._checklist_repository is not None:
                    base_path = getattr(self._project_repository, "base_path", None)
                    project_root = Path(base_path) / project_name if base_path else self._get_project_root_path(project_name)
                    if not project_root.exists():
                        project_root = self._get_project_root_path(project_name)
                    episode_title = self._get_episode_title(project_name, episode_number)
                    checklist_path = self._checklist_repository.create_episode_checklist(
                        episode_number, episode_title, project_root
                    )
                    self._checklist_repository.save_results(session, checklist_path)

                session.complete()

                self._logger_service.info(f"A31自動修正完了: セッション{session_id.value}")
                return session

            except Exception as e:
                self._logger_service.error(f"A31自動修正エラー: {e}")
                raise

    def _create_evaluation_service(self) -> A31EvaluationService:
        """Return a new instance of the A31 evaluation service."""
        # ドメインサービスは依存関係なしで生成可能
        if self._evaluation_service is None:
            self._evaluation_service = A31EvaluationService()
        return self._evaluation_service

    def _create_auto_fix_service(self) -> A31AutoFixService:
        """Return a new instance of the A31 auto fix service."""
        # ドメインサービスは依存関係なしで生成可能
        if self._auto_fix_service is None:
            self._auto_fix_service = A31AutoFixService()
        return self._auto_fix_service

    def save_fixed_content(self, session: AutoFixSession, fixed_content: str) -> None:
        """Persist the fixed manuscript content for the session.

        Args:
            session (AutoFixSession): Session describing the target file.
            fixed_content (str): Updated manuscript content.
        """
        self._unit_of_work.episode_repository.save_episode_content(session.target_file, fixed_content)

    def _get_project_root_path(self, project_name: str) -> Path:
        """Resolve the project root path for the given project name.

        Args:
            project_name (str): Project identifier.

        Returns:
            Path: Absolute project root path.
        """
        # プロジェクトリポジトリから設定を取得し、プロジェクトルートを特定
        config = self._project_repository.get_project_config(project_name)

        # 設定からプロジェクトルートを取得
        if "project_root" in config:
            return Path(config["project_root"])

        # プロジェクトリポジトリから直接プロジェクトルートを取得
        project_root = self._project_repository.get_project_root(project_name)
        if project_root and project_root.exists():
            return project_root

        # フォールバック: プロジェクトリポジトリのproject_rootから推定
        if hasattr(self._project_repository, "project_root"):
            repo_root = self._project_repository.project_root
            project_path = repo_root / project_name
            if project_path.exists():
                return project_path

        # 最終フォールバック: 現在のディレクトリからの相対パス
        return Path(f"projects/{project_name}")

    def _get_episode_title(self, project_name: str, episode_number: int) -> str:
        """Lookup the episode title from the project metadata.

        Args:
            project_name (str): Project identifier.
            episode_number (int): Episode number to inspect.

        Returns:
            str: Episode title or a fallback label when not present.
        """
        try:
            # エピソード管理データからタイトルを取得
            episode_management = self._project_repository.get_episode_management(project_name)
            episodes = episode_management.get("episodes", [])

            for episode in episodes:
                if episode.get("number") == episode_number:
                    return episode.get("title", f"第{episode_number:03d}話")

        except Exception as e:
            # エラーの場合はデフォルトタイトルを返す(ログ記録)
            self.logger.warning("エピソードタイトル取得エラー: %s", e)

        return f"第{episode_number:03d}話"

    def generate_session_report(self, session: AutoFixSession) -> str:
        """セッションレポートの生成

        Args:
            session: 修正セッション

        Returns:
            str: レポート文字列
        """
        report_lines = [
            "=" * 50,
            "A31自動修正セッション完了",
            "=" * 50,
            f"セッションID: {session.session_id.value}",
            f"対象ファイル: {session.target_file}",
            f"修正レベル: {session.fix_level.value}",
            f"処理時間: {session.get_duration()}秒",
            "",
            "📊 修正結果サマリー:",
            f"  - 成功: {session.get_successful_fixes_count()}件",
            f"  - 失敗: {session.get_failed_fixes_count()}件",
            f"  - 総改善点: {session.get_total_improvement():.1f}点",
            "",
        ]

        # 個別修正結果の詳細
        if session.results:
            report_lines.append("🔧 修正詳細:")
            for result in session.results:
                status = "✅" if result.fix_applied else "❌"
                report_lines.append(f"  {status} {result.item_id} ({result.fix_type})")

                if result.fix_applied and result.changes_made:
                    report_lines.extend(f"    - {change}" for change in result.changes_made)

                    improvement = result.after_score - result.before_score
                    if improvement > 0:
                        report_lines.append(f"    💡 スコア改善: {improvement:.1f}点")

                report_lines.append("")

        # 改善提案の追加
        auto_fix_service = self._create_auto_fix_service()
        suggestions = auto_fix_service.generate_improvement_suggestions(
            self._get_evaluation_results_from_session(session), self._get_checklist_items_from_session(session)
        )

        if suggestions:
            report_lines.append("💭 改善提案:")
            report_lines.extend(f"  - {suggestion}" for suggestion in suggestions)
            report_lines.append("")

        report_lines.append("=" * 50)

        return "\n".join(report_lines)

    def _get_episode_file_path(self, project_name: str, episode_number: int) -> Path:
        """エピソードファイルパスの取得"""
        # 共通基盤の命名規則に統一
        try:
            path_service = self.get_path_service()
            return path_service.get_manuscript_path(episode_number)
        except Exception:
            # フォールバック: 旧来のパス（互換用）
            return Path(f"{project_name}/40_原稿/第{episode_number:03d}話_タイトル.md")

    def _get_project_metadata(self, _project_name: str) -> dict[str, any]:
        """プロジェクトメタデータの取得"""
        # 実際の実装では、プロジェクト設定やキャラクター情報等を取得
        return {"quality_score": 75.0, "characters": {}, "terminology": {}}

    def _get_evaluation_results_from_session(self, session: AutoFixSession) -> dict[str, EvaluationResult]:
        """セッションから評価結果を復元"""
        # 実際の実装では、セッションに保存された評価結果を取得
        # ここでは簡易的な実装
        evaluation_results = {}

        for result in session.results:
            evaluation_results[result.item_id] = EvaluationResult.create_failed_result(
                item_id=result.item_id,
                current_score=result.before_score,
                threshold_value=result.after_score,
                details={},
            )

        return evaluation_results

    def _get_checklist_items_from_session(self, _session: AutoFixSession) -> list[A31ChecklistItem]:
        """セッションからチェックリスト項目を復元"""
        # 実際の実装では、より詳細な復元処理が必要
        # ここでは空のリストを返す
        return []

class A31AutoFixUseCaseError(Exception):
    """A31自動修正ユースケースエラー"""

class EpisodeNotFoundError(A31AutoFixUseCaseError):
    """エピソード未発見エラー"""

class InvalidChecklistItemError(A31AutoFixUseCaseError):
    """無効なチェックリスト項目エラー"""

class FixLevelNotSupportedError(A31AutoFixUseCaseError):
    """修正レベル未対応エラー"""



"""プロジェクト初期化ドメイン - 値オブジェクトのテスト

TDD準拠テスト:
    - Genre (Enum)
- WritingStyle (Enum)
- UpdateFrequency (Enum)
- InitializationConfig


仕様書: SPEC-UNIT-TEST
"""

import pytest

from noveler.domain.exceptions import DomainException
from noveler.domain.initialization.value_objects import (
    Genre,
    InitializationConfig,
    UpdateFrequency,
    WritingStyle,
)


class TestGenre:
    """Genreエンティティ(Enum)のテストクラス"""

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GENRE_VALUES")
    def test_genre_values(self) -> None:
        """ジャンル値テスト"""
        assert Genre.FANTASY.value == "fantasy"
        assert Genre.ROMANCE.value == "romance"
        assert Genre.MYSTERY.value == "mystery"
        assert Genre.SLICE_OF_LIFE.value == "slice_of_life"
        assert Genre.SCIENCE_FICTION.value == "science_fiction"

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GENRE_ENUM_COUNT")
    def test_genre_enum_count(self) -> None:
        """ジャンル数テスト"""
        assert len(Genre) == 5

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GENRE_MEMBERSHIP")
    def test_genre_membership(self) -> None:
        """ジャンルメンバーシップテスト"""
        assert Genre.FANTASY in Genre
        assert Genre.ROMANCE in Genre
        assert Genre.MYSTERY in Genre
        assert Genre.SLICE_OF_LIFE in Genre
        assert Genre.SCIENCE_FICTION in Genre

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GENRE_LEGACY_SUPPORT")
    def test_genre_legacy_members_excluded_from_iteration(self) -> None:
        """レガシージャンルが列挙に含まれないことを確認"""
        canonical_members = list(Genre)

        assert Genre.HORROR not in canonical_members
        assert Genre.LITERARY not in canonical_members
        assert Genre.OTHER not in canonical_members
        # SCIFI は SCIENCE_FICTION の別名として扱われる
        assert Genre.SCIENCE_FICTION in canonical_members


class TestWritingStyle:
    """WritingStyleエンティティ(Enum)のテストクラス"""

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-WRITING_STYLE_VALUES")
    def test_writing_style_values(self) -> None:
        """執筆スタイル値テスト"""
        assert WritingStyle.LIGHT.value == "light"
        assert WritingStyle.SERIOUS.value == "serious"
        assert WritingStyle.COMEDY.value == "comedy"
        assert WritingStyle.DARK.value == "dark"

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-WRITING_STYLE_ENUM_C")
    def test_writing_style_enum_count(self) -> None:
        """執筆スタイル数テスト"""
        assert len(WritingStyle) == 4

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-WRITING_STYLE_MEMBER")
    def test_writing_style_membership(self) -> None:
        """執筆スタイルメンバーシップテスト"""
        assert WritingStyle.LIGHT in WritingStyle
        assert WritingStyle.SERIOUS in WritingStyle
        assert WritingStyle.COMEDY in WritingStyle
        assert WritingStyle.DARK in WritingStyle


class TestUpdateFrequency:
    """UpdateFrequencyエンティティ(Enum)のテストクラス"""

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-UPDATE_FREQUENCY_VAL")
    def test_update_frequency_values(self) -> None:
        """更新頻度値テスト"""
        assert UpdateFrequency.DAILY.value == "daily"
        assert UpdateFrequency.WEEKLY.value == "weekly"
        assert UpdateFrequency.BIWEEKLY.value == "biweekly"
        assert UpdateFrequency.MONTHLY.value == "monthly"

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-UPDATE_FREQUENCY_ENU")
    def test_update_frequency_enum_count(self) -> None:
        """更新頻度数テスト"""
        assert len(UpdateFrequency) == 4

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-UPDATE_FREQUENCY_MEM")
    def test_update_frequency_membership(self) -> None:
        """更新頻度メンバーシップテスト"""
        assert UpdateFrequency.DAILY in UpdateFrequency
        assert UpdateFrequency.WEEKLY in UpdateFrequency
        assert UpdateFrequency.BIWEEKLY in UpdateFrequency
        assert UpdateFrequency.MONTHLY in UpdateFrequency


class TestInitializationConfig:
    """InitializationConfig値オブジェクトのテストクラス"""

    @pytest.fixture
    def valid_config(self) -> InitializationConfig:
        """有効な初期化設定"""
        return InitializationConfig(
            genre=Genre.FANTASY,
            writing_style=WritingStyle.LIGHT,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name="転生したら最強の魔法使いだった件",
            author_name="テスト作家",
        )

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-INITIALIZATION_CONFI")
    def test_initialization_config_creation_valid(self, valid_config: InitializationConfig) -> None:
        """有効な値での初期化設定作成テスト"""
        assert valid_config.genre == Genre.FANTASY
        assert valid_config.writing_style == WritingStyle.LIGHT
        assert valid_config.update_frequency == UpdateFrequency.WEEKLY
        assert valid_config.project_name == "転生したら最強の魔法使いだった件"
        assert valid_config.author_name == "テスト作家"

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-INITIALIZATION_CONFI")
    def test_initialization_config_is_frozen(self, valid_config: InitializationConfig) -> None:
        """設定オブジェクトの不変性テスト"""
        with pytest.raises(AttributeError, match=".*"):
            valid_config.project_name = "新しい名前"  # type: ignore

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-VALIDATE_PROJECT_NAM")
    def test_validate_project_name_empty_error(self) -> None:
        """空のプロジェクト名エラーテスト"""
        with pytest.raises(DomainException, match="プロジェクト名は1文字以上50文字以下で入力してください"):
            InitializationConfig(
                genre=Genre.FANTASY,
                writing_style=WritingStyle.LIGHT,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name="",
                author_name="テスト作家",
            )

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-VALIDATE_PROJECT_NAM")
    def test_validate_project_name_whitespace_only_error(self) -> None:
        """空白のみのプロジェクト名エラーテスト"""
        with pytest.raises(DomainException, match="プロジェクト名は1文字以上50文字以下で入力してください"):
            InitializationConfig(
                genre=Genre.FANTASY,
                writing_style=WritingStyle.LIGHT,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name="   ",
                author_name="テスト作家",
            )

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-VALIDATE_PROJECT_NAM")
    def test_validate_project_name_too_long_error(self) -> None:
        """長すぎるプロジェクト名エラーテスト"""
        long_name = "あ" * 51  # 51文字
        with pytest.raises(DomainException, match="プロジェクト名は1文字以上50文字以下で入力してください"):
            InitializationConfig(
                genre=Genre.FANTASY,
                writing_style=WritingStyle.LIGHT,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name=long_name,
                author_name="テスト作家",
            )

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-VALIDATE_PROJECT_NAM")
    def test_validate_project_name_max_length_valid(self) -> None:
        """最大文字数のプロジェクト名有効テスト"""
        max_name = "あ" * 50  # 50文字(上限)
        config = InitializationConfig(
            genre=Genre.FANTASY,
            writing_style=WritingStyle.LIGHT,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name=max_name,
            author_name="テスト作家",
        )

        assert config.project_name == max_name

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-VALIDATE_PROJECT_NAM")
    def test_validate_project_name_invalid_chars_error(self) -> None:
        """無効文字を含むプロジェクト名エラーテスト"""
        invalid_chars = ["<", ">", ":", '"', "/", "\\", "|", "?", "*"]

        for char in invalid_chars:
            invalid_name = f"プロジェクト{char}名前"
            with pytest.raises(DomainException, match="プロジェクト名に使用できない文字が含まれています"):
                InitializationConfig(
                    genre=Genre.FANTASY,
                    writing_style=WritingStyle.LIGHT,
                    update_frequency=UpdateFrequency.WEEKLY,
                    project_name=invalid_name,
                    author_name="テスト作家",
                )

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-VALIDATE_PROJECT_NAM")
    def test_validate_project_name_valid_chars_success(self) -> None:
        """有効文字を含むプロジェクト名成功テスト"""
        valid_chars = ["ー", "!", "？", "(", ")", "【", "】", "〜", "・"]

        for char in valid_chars:
            valid_name = f"プロジェクト{char}名前"
            config = InitializationConfig(
                genre=Genre.FANTASY,
                writing_style=WritingStyle.LIGHT,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name=valid_name,
                author_name="テスト作家",
            )

            assert char in config.project_name

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-VALIDATE_AUTHOR_NAME")
    def test_validate_author_name_empty_error(self) -> None:
        """空の作者名エラーテスト"""
        with pytest.raises(DomainException, match="作者名は必須です"):
            InitializationConfig(
                genre=Genre.FANTASY,
                writing_style=WritingStyle.LIGHT,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name="テストプロジェクト",
                author_name="",
            )

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-VALIDATE_AUTHOR_NAME")
    def test_validate_author_name_whitespace_only_error(self) -> None:
        """空白のみの作者名エラーテスト"""
        with pytest.raises(DomainException, match="作者名は必須です"):
            InitializationConfig(
                genre=Genre.FANTASY,
                writing_style=WritingStyle.LIGHT,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name="テストプロジェクト",
                author_name="   ",
            )

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-VALIDATE_AUTHOR_NAME")
    def test_validate_author_name_valid(self) -> None:
        """有効な作者名テスト"""
        valid_names = ["テスト作家", "Test Author", "作家123", "ペンネーム@site"]

        for name in valid_names:
            config = InitializationConfig(
                genre=Genre.FANTASY,
                writing_style=WritingStyle.LIGHT,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name="テストプロジェクト",
                author_name=name,
            )

            assert config.author_name == name

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-IS_VALID_TRUE")
    def test_is_valid_true(self, valid_config: InitializationConfig) -> None:
        """有効性チェック(True)テスト"""
        assert valid_config.is_valid() is True

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-IS_VALID_FALSE_INVAL")
    def test_is_valid_false_invalid_project_name(self) -> None:
        """有効性チェック(False・無効プロジェクト名)テスト"""
        # NOTE: 現在の実装では、無効な設定で作成時に例外が発生するため、
        # このテストは実行されません。実装を修正する必要があります。
        # ここでは設計意図を示すためのテストとして残します。

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-IS_COMPATIBLE_WITH_S")
    def test_is_compatible_with_style_dark_comedy_false(self) -> None:
        """スタイル互換性(ダーク・コメディ非互換)テスト"""
        dark_config = InitializationConfig(
            genre=Genre.FANTASY,
            writing_style=WritingStyle.DARK,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name="ダークファンタジー",
            author_name="ダーク作家",
        )

        assert dark_config.is_compatible_with_style(WritingStyle.COMEDY) is False

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-IS_COMPATIBLE_WITH_S")
    def test_is_compatible_with_style_comedy_dark_false(self) -> None:
        """スタイル互換性(コメディ・ダーク非互換)テスト"""
        comedy_config = InitializationConfig(
            genre=Genre.SLICE_OF_LIFE,
            writing_style=WritingStyle.COMEDY,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name="コメディ小説",
            author_name="コメディ作家",
        )

        assert comedy_config.is_compatible_with_style(WritingStyle.DARK) is False

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-IS_COMPATIBLE_WITH_S")
    def test_is_compatible_with_style_compatible_combinations(self) -> None:
        """スタイル互換性(互換性のある組み合わせ)テスト"""
        test_cases = [
            (WritingStyle.LIGHT, WritingStyle.SERIOUS),
            (WritingStyle.LIGHT, WritingStyle.COMEDY),
            (WritingStyle.SERIOUS, WritingStyle.LIGHT),
            (WritingStyle.COMEDY, WritingStyle.LIGHT),
            (WritingStyle.DARK, WritingStyle.SERIOUS),
            (WritingStyle.SERIOUS, WritingStyle.DARK),
        ]

        for current_style, test_style in test_cases:
            config = InitializationConfig(
                genre=Genre.FANTASY,
                writing_style=current_style,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name="テストプロジェクト",
                author_name="テスト作家",
            )

            assert config.is_compatible_with_style(test_style) is True

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-IS_COMPATIBLE_WITH_S")
    def test_is_compatible_with_style_self_compatibility(self) -> None:
        """スタイル互換性(自己互換性)テスト"""
        for style in WritingStyle:
            config = InitializationConfig(
                genre=Genre.FANTASY,
                writing_style=style,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name="テストプロジェクト",
                author_name="テスト作家",
            )

            assert config.is_compatible_with_style(style) is True

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_default(self, valid_config: InitializationConfig) -> None:
        """推奨設定取得(デフォルト)テスト"""
        settings = valid_config.get_recommended_settings()

        assert "target_episode_length" in settings
        assert "chapters_per_arc" in settings
        assert "quality_focus_areas" in settings

        assert settings["chapters_per_arc"] == 10
        assert "readability" in settings["quality_focus_areas"]
        assert "engagement" in settings["quality_focus_areas"]

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_fantasy_genre(self) -> None:
        """推奨設定取得(ファンタジージャンル)テスト"""
        config = InitializationConfig(
            genre=Genre.FANTASY,
            writing_style=WritingStyle.SERIOUS,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name="ファンタジー小説",
            author_name="ファンタジー作家",
        )

        settings = config.get_recommended_settings()

        assert settings["target_episode_length"] == 4000  # ファンタジーは長め
        assert "world_building" in settings["quality_focus_areas"]

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_mystery_genre(self) -> None:
        """推奨設定取得(ミステリージャンル)テスト"""
        config = InitializationConfig(
            genre=Genre.MYSTERY,
            writing_style=WritingStyle.SERIOUS,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name="ミステリー小説",
            author_name="ミステリー作家",
        )

        settings = config.get_recommended_settings()

        assert "logical_consistency" in settings["quality_focus_areas"]
        assert "tension" in settings["quality_focus_areas"]

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_romance_genre(self) -> None:
        """推奨設定取得(ロマンスジャンル)テスト"""
        config = InitializationConfig(
            genre=Genre.ROMANCE,
            writing_style=WritingStyle.LIGHT,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name="ロマンス小説",
            author_name="ロマンス作家",
        )

        settings = config.get_recommended_settings()

        assert "emotional_depth" in settings["quality_focus_areas"]
        assert "character_development" in settings["quality_focus_areas"]

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_light_style(self) -> None:
        """推奨設定取得(ライトスタイル)テスト"""
        config = InitializationConfig(
            genre=Genre.SLICE_OF_LIFE,
            writing_style=WritingStyle.LIGHT,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name="日常系小説",
            author_name="日常系作家",
        )

        settings = config.get_recommended_settings()

        # ライトスタイルは短めに調整される
        assert settings["target_episode_length"] <= 3500

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_serious_style(self) -> None:
        """推奨設定取得(シリアススタイル)テスト"""
        config = InitializationConfig(
            genre=Genre.SLICE_OF_LIFE,
            writing_style=WritingStyle.SERIOUS,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name="シリアス小説",
            author_name="シリアス作家",
        )

        settings = config.get_recommended_settings()

        # シリアススタイルは長めに調整される
        assert settings["target_episode_length"] >= 3500

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_fantasy_serious_combination(self) -> None:
        """推奨設定取得(ファンタジー・シリアス組み合わせ)テスト"""
        config = InitializationConfig(
            genre=Genre.FANTASY,
            writing_style=WritingStyle.SERIOUS,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name="シリアスファンタジー",
            author_name="ファンタジー作家",
        )

        settings = config.get_recommended_settings()

        # ファンタジー(4000文字)× シリアス(3500文字以上)= max(4000, 3500) = 4000
        assert settings["target_episode_length"] == 4000
        assert "world_building" in settings["quality_focus_areas"]

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_romance_light_combination(self) -> None:
        """推奨設定取得(ロマンス・ライト組み合わせ)テスト"""
        config = InitializationConfig(
            genre=Genre.ROMANCE,
            writing_style=WritingStyle.LIGHT,
            update_frequency=UpdateFrequency.WEEKLY,
            project_name="ライトロマンス",
            author_name="ロマンス作家",
        )

        settings = config.get_recommended_settings()

        # ロマンス(3000文字)× ライト(3500文字以下)= min(3000, 3500) = 3000
        assert settings["target_episode_length"] == 3000
        assert "emotional_depth" in settings["quality_focus_areas"]
        assert "character_development" in settings["quality_focus_areas"]

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_all_genres(self) -> None:
        """推奨設定取得(全ジャンル)テスト"""
        for genre in Genre:
            config = InitializationConfig(
                genre=genre,
                writing_style=WritingStyle.LIGHT,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name=f"{genre.value}小説",
                author_name="テスト作家",
            )

            settings = config.get_recommended_settings()

            # 全ジャンルで基本項目が含まれる
            assert "target_episode_length" in settings
            assert "chapters_per_arc" in settings
            assert "quality_focus_areas" in settings
            assert isinstance(settings["target_episode_length"], int)
            assert settings["target_episode_length"] > 0

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_all_styles(self) -> None:
        """推奨設定取得(全スタイル)テスト"""
        for style in WritingStyle:
            config = InitializationConfig(
                genre=Genre.SLICE_OF_LIFE,
                writing_style=style,
                update_frequency=UpdateFrequency.WEEKLY,
                project_name=f"{style.value}小説",
                author_name="テスト作家",
            )

            settings = config.get_recommended_settings()

            # 全スタイルで基本項目が含まれる
            assert "target_episode_length" in settings
            assert "chapters_per_arc" in settings
            assert "quality_focus_areas" in settings

            # スタイル別の文字数調整確認
            if style == WritingStyle.LIGHT:
                assert settings["target_episode_length"] <= 3500
            elif style == WritingStyle.SERIOUS:
                assert settings["target_episode_length"] >= 3500

    @pytest.mark.spec("SPEC-INITIALIZATION_VALUE_OBJECTS-GET_RECOMMENDED_SETT")
    def test_get_recommended_settings_immutability(self, valid_config: InitializationConfig) -> None:
        """推奨設定の不変性テスト"""
        settings1 = valid_config.get_recommended_settings()
        settings2 = valid_config.get_recommended_settings()

        # 異なるオブジェクトが返される(変更が他に影響しない)
        assert settings1 is not settings2
        assert settings1 == settings2

        # 変更が他の呼び出しに影響しない
        settings1["target_episode_length"] = 9999
        settings3 = valid_config.get_recommended_settings()
        assert settings3["target_episode_length"] != 9999



#!/usr/bin/env python3

"""Application.use_cases.a31_claude_code_evaluation_use_case
Where: Application use case orchestrating Claude Code evaluations for A31 checklists.
What: Resolves dependencies, retrieves checklist items, runs evaluations, and formats results.
Why: Enables automated A31 assessments inside Claude Code sessions without manual setup.
"""

from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from noveler.domain.interfaces.logger_service_protocol import ILoggerService
    from noveler.infrastructure.unit_of_work import IUnitOfWork

from noveler.application.base.abstract_use_case import AbstractUseCase
from noveler.domain.entities.a31_checklist_item import A31ChecklistItem
from noveler.domain.interfaces import IConsoleService, IPathService
from noveler.domain.repositories.a31_checklist_repository import A31ChecklistRepository
from noveler.domain.services.claude_code_evaluation_service import (
    ClaudeCodeEvaluationRequest,
    ClaudeCodeEvaluationService,
)
from noveler.domain.services.evaluation_result_formatter import EvaluationResultFormatter
from noveler.domain.services.project_config_provider import ProjectConfigProvider
from noveler.domain.services.project_path_resolver import ProjectPathResolver
from noveler.domain.value_objects.a31_evaluation_result import EvaluationResult
from noveler.domain.value_objects.episode_number import EpisodeNumber
from noveler.domain.value_objects.file_path import FilePath


@dataclass(frozen=True)
class A31ClaudeCodeEvaluationRequest:
    """A31 Claude Code評価ユースケースリクエスト"""

    project_name: str
    episode_number: int
    include_a31: bool = True
    a31_only: bool = False
    show_review: bool = False
    review_verbosity: str = "standard"  # none/basic/standard/verbose


@dataclass
class A31ClaudeCodeEvaluationResponse:
    """A31 Claude Code評価ユースケースレスポンス"""

    success: bool
    project_name: str
    episode_number: int
    total_items: int
    evaluated_items: int
    passed_items: int
    evaluation_results: dict[str, EvaluationResult]
    checklist_file_path: str | None = None
    error_message: str | None = None
    review_output: str | None = None


class A31ClaudeCodeEvaluationUseCase(AbstractUseCase[A31ClaudeCodeEvaluationRequest, A31ClaudeCodeEvaluationResponse]):
    """A31 Claude Code連携評価ユースケース

    Claude Code実行中セッションでの自動評価機能を提供。
    既存コマンド(check, complete, a31-auto-fix)との統合。
    """

    def __init__(self,
        logger_service: ILoggerService = None,
        unit_of_work: IUnitOfWork = None,
        console_service: IConsoleService | None = None,
        path_service: IPathService | None = None,
        **kwargs) -> None:
        """初期化

        Args:
            console_service: コンソールサービス（DI注入）
            path_service: パスサービス（DI注入）
            logger: ロガー（DI注入）
            config_service: 設定サービス（DI注入）
            repository_factory: リポジトリファクトリー（DI注入）
        """
        super().__init__(
            console_service=console_service,
            path_service=path_service,
            logger_service=logger_service,
            **kwargs)
        # B20準拠: 標準DIサービス
        self._logger_service = logger_service
        self._unit_of_work = unit_of_work

        # 遅延初期化によるサービス取得
        self._a31_checklist_repository = None
        self._claude_code_evaluation_service = None
        self._project_config_provider = None
        self._project_path_resolver = None
        self._evaluation_result_formatter = None
        self._a31_auto_fix_use_case = None

    @property
    def a31_checklist_repository(self) -> A31ChecklistRepository:
        """A31チェックリストリポジトリ取得"""
        if self._a31_checklist_repository is None:
            self._a31_checklist_repository = self.repository_factory.get_a31_checklist_repository()
        return self._a31_checklist_repository

    @property
    def claude_code_evaluation_service(self) -> ClaudeCodeEvaluationService:
        """Claude Code評価サービス取得"""
        if self._claude_code_evaluation_service is None:
            try:
                from noveler.domain.services.claude_code_evaluation_service import ClaudeCodeEvaluationService
                from noveler.infrastructure.di.container import resolve_service

                self._claude_code_evaluation_service = resolve_service(ClaudeCodeEvaluationService)
            except (ImportError, ValueError):
                from noveler.domain.services.claude_code_evaluation_service import ClaudeCodeEvaluationService

                self._claude_code_evaluation_service = ClaudeCodeEvaluationService()
        return self._claude_code_evaluation_service

    @property
    def project_config_provider(self) -> ProjectConfigProvider:
        """プロジェクト設定プロバイダ取得"""
        if self._project_config_provider is None:
            try:
                from noveler.domain.services.project_config_provider import ProjectConfigProvider
                from noveler.infrastructure.di.container import resolve_service

                self._project_config_provider = resolve_service(ProjectConfigProvider)
            except (ImportError, ValueError):
                from noveler.domain.services.project_config_provider import ProjectConfigProvider

                self._project_config_provider = ProjectConfigProvider()
        return self._project_config_provider

    @property
    def project_path_resolver(self) -> ProjectPathResolver:
        """プロジェクトパス解決サービス取得"""
        if self._project_path_resolver is None:
            try:
                from noveler.domain.services.project_path_resolver import ProjectPathResolver
                from noveler.infrastructure.di.container import resolve_service

                self._project_path_resolver = resolve_service(ProjectPathResolver)
            except (ImportError, ValueError):
                from noveler.domain.services.project_path_resolver import ProjectPathResolver

                self._project_path_resolver = ProjectPathResolver()
        return self._project_path_resolver

    @property
    def evaluation_result_formatter(self) -> EvaluationResultFormatter:
        """評価結果フォーマッタ取得"""
        if self._evaluation_result_formatter is None:
            try:
                from noveler.domain.services.evaluation_result_formatter import EvaluationResultFormatter
                from noveler.infrastructure.di.container import resolve_service

                self._evaluation_result_formatter = resolve_service(EvaluationResultFormatter)
            except (ImportError, ValueError):
                from noveler.domain.services.evaluation_result_formatter import EvaluationResultFormatter

                self._evaluation_result_formatter = EvaluationResultFormatter()
        return self._evaluation_result_formatter

    async def execute(
        self,
        request: A31ClaudeCodeEvaluationRequest,
    ) -> A31ClaudeCodeEvaluationResponse:
        """A31 Claude Code連携評価を実行

        Args:
            request: 評価リクエスト

        Returns:
            A31ClaudeCodeEvaluationResponse: 評価結果
        """
        try:
            self.logger.info(
                "A31評価開始: project=%s, episode=%s",
                request.project_name,
                request.episode_number,
            )

            # 1. プロジェクト設定取得
            config = self.project_config_provider.get_project_config(request.project_name)
            if not self.project_config_provider.should_run_a31_evaluation(
                config, request.include_a31, request.a31_only
            ):
                return self._create_skipped_response(request)

            # 2. チェックリスト取得または作成
            checklist_items = await self._get_or_create_checklist(
                request.project_name,
                request.episode_number,
            )

            # 3. 評価対象項目の決定
            target_items = self._determine_target_items(checklist_items)

            # 4. Claude Code評価実行
            evaluation_results = await self._execute_claude_code_evaluation(
                request.project_name,
                request.episode_number,
                target_items,
            )

            # 5. チェックリスト更新
            checklist_file_path = await self._update_checklist_file(
                request.project_name,
                request.episode_number,
                evaluation_results,
            )

            # 6. レビュー出力生成
            review_output = None
            if request.show_review:
                review_output = self.evaluation_result_formatter.generate_review_output(
                    evaluation_results,
                    request.review_verbosity,
                )

            # 7. レスポンス作成
            passed_count = sum(1 for result in evaluation_results.values() if result.passed)

            self.logger.info(
                "A31評価完了: total=%s, evaluated=%s, passed=%s",
                len(target_items),
                len(evaluation_results),
                passed_count,
            )

            return A31ClaudeCodeEvaluationResponse(
                success=True,
                project_name=request.project_name,
                episode_number=request.episode_number,
                total_items=len(target_items),
                evaluated_items=len(evaluation_results),
                passed_items=passed_count,
                evaluation_results=evaluation_results,
                checklist_file_path=checklist_file_path,
                review_output=review_output,
            )

        except Exception as e:
            self.logger.error(
                "A31評価の予期しないエラー: %s: %s",
                type(e).__name__,
                e,
                exc_info=True,
            )
            return A31ClaudeCodeEvaluationResponse(
                success=False,
                project_name=request.project_name,
                episode_number=request.episode_number,
                total_items=0,
                evaluated_items=0,
                passed_items=0,
                evaluation_results={},
                error_message=f"A31評価エラー: {e!s}",
            )

    def _create_skipped_response(
        self,
        request: A31ClaudeCodeEvaluationRequest,
    ) -> A31ClaudeCodeEvaluationResponse:
        """スキップされた場合のレスポンスを作成

        Args:
            request: 評価リクエスト

        Returns:
            A31ClaudeCodeEvaluationResponse: スキップレスポンス
        """
        return A31ClaudeCodeEvaluationResponse(
            success=True,
            project_name=request.project_name,
            episode_number=request.episode_number,
            total_items=0,
            evaluated_items=0,
            passed_items=0,
            evaluation_results={},
            error_message="A31評価がスキップされました(設定により無効化)",
        )

    async def _get_or_create_checklist(
        self,
        project_name: str,
        episode_number: int,
    ) -> list[A31ChecklistItem]:
        """チェックリストを取得または作成

        Args:
            project_name: プロジェクト名
            episode_number: エピソード番号

        Returns:
            list[A31ChecklistItem]: チェックリスト項目
        """
        try:
            # 既存チェックリストの取得を試行
            return await self._unit_of_work.a31_checklist_repository.get_checklist_items(
                project_name,
                EpisodeNumber(episode_number),
            )
        except (FileNotFoundError, ValueError, TypeError):
            # 存在しない場合はテンプレートから作成
            return await self._unit_of_work.a31_checklist_repository.create_from_template(
                project_name,
                EpisodeNumber(episode_number),
            )

    def _determine_target_items(
        self,
        checklist_items: list[A31ChecklistItem],
    ) -> list[A31ChecklistItem]:
        """評価対象項目を決定

        Args:
            checklist_items: 全チェックリスト項目

        Returns:
            list[A31ChecklistItem]: 評価対象項目
        """
        # Claude Code評価可能項目のみを抽出
        claude_evaluable_types = {
            "document_review",
            "content_review",
            "content_planning",
            "risk_assessment",
            "scene_design",
        }

        return [item for item in checklist_items if item.item_type.value in claude_evaluable_types]

    async def _execute_claude_code_evaluation(
        self,
        project_name: str,
        episode_number: int,
        target_items: list[A31ChecklistItem],
    ) -> dict[str, EvaluationResult]:
        """Claude Code評価を実行

        Args:
            project_name: プロジェクト名
            episode_number: エピソード番号
            target_items: 評価対象項目

        Returns:
            dict[str, EvaluationResult]: 項目ID別の評価結果
        """
        results: dict[str, Any] = {}

        # エピソードファイルパスの決定
        episode_file_path = self.project_path_resolver.resolve_episode_file_path(project_name, episode_number)

        for item in target_items:
            try:
                # 関連ファイルの決定(簡略化版)
                context_files: list[FilePath] = []

                # 評価リクエスト作成
                eval_request = ClaudeCodeEvaluationRequest(
                    item=item,
                    episode_file_path=episode_file_path,
                    context_files=context_files,
                    metadata={"project_name": project_name, "episode_number": episode_number},
                )

                # Claude Code評価実行
                result = await self.claude_code_evaluation_service.evaluate_item(eval_request)
                results[item.item_id] = result

            except (ConnectionError, TimeoutError, ValueError, AttributeError, ImportError) as e:
                self.logger.warning(
                    "評価項目 %s でエラー: %s",
                    item.item_id,
                    e,
                )
                # エラー時は失敗結果を作成
                results[item.item_id] = EvaluationResult(
                    item_id=item.item_id,
                    current_score=0.0,
                    threshold_value=item.threshold.value if item.threshold else 0.0,
                    passed=False,
                    details={"error": f"評価エラー: {e!s}"},
                )

        return results

    async def _update_checklist_file(
        self,
        project_name: str,
        episode_number: int,
        evaluation_results: dict[str, EvaluationResult],
    ) -> str:
        """チェックリストファイルを更新

        Args:
            project_name: プロジェクト名
            episode_number: エピソード番号
            evaluation_results: 評価結果

        Returns:
            str: 更新されたチェックリストファイルパス
        """
        return await self._unit_of_work.a31_checklist_repository.update_evaluation_results(
            project_name,
            EpisodeNumber(episode_number),
            evaluation_results,
        )



#!/usr/bin/env python3
"""DDD品質ゲート

仕様書: SPEC-DDD-AUTO-COMPLIANCE-001
CI/CDパイプライン統合用DDD品質ゲート実装
"""

import argparse
import asyncio
import os
import sys
from dataclasses import dataclass
from enum import Enum
from pathlib import Path
from typing import Any

from noveler.domain.interfaces.i_unified_file_storage import FileContentType
from noveler.domain.services.architecture_dependency_analyzer import ArchitectureDependencyAnalyzer
from noveler.infrastructure.logging.unified_logger import get_logger
from noveler.infrastructure.services.ddd_compliance_engine import (
    DDDComplianceEngine,
    ValidationLevel,
    ViolationSeverity,
)
from noveler.infrastructure.storage import UnifiedFileStorageService
from noveler.presentation.shared.shared_utilities import console


class QualityGateMode(Enum):
    """品質ゲートモード"""

    STRICT = "strict"  # 厳格モード（CI/CDでの使用）
    MODERATE = "moderate"  # 中程度モード（開発中の使用）
    REPORT_ONLY = "report"  # レポートのみ（情報収集）
    CI = "ci"  # CI環境専用


class ExitCode(Enum):
    """終了コード"""

    SUCCESS = 0
    WARNINGS = 1
    VIOLATIONS = 2
    ERROR = 3


@dataclass
class QualityGateResult:
    """品質ゲート結果"""

    mode: QualityGateMode
    passed: bool
    exit_code: ExitCode
    compliance_percentage: float
    violations_count: int
    critical_violations: int
    warnings_count: int
    report_path: str | None
    summary: dict[str, Any]


class DDDQualityGate:
    """DDD品質ゲート

    責務:
        - CI/CDパイプライン統合
        - 品質基準による合否判定
        - 詳細レポート生成
        - 段階的品質評価
        - Git Hooks統合

    設計原則:
        - 高速実行（CI環境での制約）
        - 明確な合否基準
        - 詳細なフィードバック
    """

    def __init__(self, project_root: Path, mode: QualityGateMode = QualityGateMode.MODERATE) -> None:
        """初期化

        Args:
            project_root: プロジェクトルートパス
            mode: 品質ゲートモード
        """
        self.project_root = project_root
        self.mode = mode
        self.logger = get_logger(__name__)

        # 品質基準設定
        self._initialize_quality_standards()

        # コンポーネント初期化
        self.compliance_engine = DDDComplianceEngine(project_root, self._get_validation_level())

        self.dependency_analyzer = ArchitectureDependencyAnalyzer(project_root)

        # CI環境検出
        self.is_ci_environment = self._detect_ci_environment()

    def _initialize_quality_standards(self) -> None:
        """品質基準の初期化"""
        self.quality_standards = {
            QualityGateMode.STRICT: {
                "min_compliance_percentage": 95.0,
                "max_critical_violations": 0,
                "max_high_violations": 2,
                "max_total_violations": 10,
                "require_architecture_health": 0.9,
            },
            QualityGateMode.MODERATE: {
                "min_compliance_percentage": 85.0,
                "max_critical_violations": 2,
                "max_high_violations": 5,
                "max_total_violations": 20,
                "require_architecture_health": 0.7,
            },
            QualityGateMode.REPORT_ONLY: {
                "min_compliance_percentage": 0.0,
                "max_critical_violations": float("inf"),
                "max_high_violations": float("inf"),
                "max_total_violations": float("inf"),
                "require_architecture_health": 0.0,
            },
            QualityGateMode.CI: {
                "min_compliance_percentage": 90.0,
                "max_critical_violations": 1,
                "max_high_violations": 3,
                "max_total_violations": 15,
                "require_architecture_health": 0.8,
            },
        }

    def _get_validation_level(self) -> ValidationLevel:
        """検証レベル取得"""
        mapping = {
            QualityGateMode.STRICT: ValidationLevel.STRICT,
            QualityGateMode.CI: ValidationLevel.STRICT,
            QualityGateMode.MODERATE: ValidationLevel.MODERATE,
            QualityGateMode.REPORT_ONLY: ValidationLevel.BASIC,
        }
        return mapping.get(self.mode, ValidationLevel.MODERATE)

    def _detect_ci_environment(self) -> bool:
        """CI環境検出"""
        ci_indicators = [
            "CI",
            "CONTINUOUS_INTEGRATION",
            "GITHUB_ACTIONS",
            "GITLAB_CI",
            "JENKINS_URL",
            "TRAVIS",
            "CIRCLECI",
            "AZURE_DEVOPS",
        ]

        return any(os.getenv(indicator) for indicator in ci_indicators)

    async def run_quality_gate(self) -> QualityGateResult:
        """品質ゲート実行

        Returns:
            QualityGateResult: 品質ゲート結果
        """
        self.logger.info("DDD品質ゲート実行開始 - モード: %s", self.mode.value)

        try:
            # 1. DDD準拠性分析
            compliance_report = await self.compliance_engine.analyze_project_compliance()

            # 2. アーキテクチャ依存関係分析
            architecture_result = await self.dependency_analyzer.analyze_project_architecture()

            # 3. 品質評価
            quality_result = self._evaluate_quality(compliance_report, architecture_result)

            # 4. レポート生成
            report_path = await self._generate_quality_report(compliance_report, architecture_result, quality_result)

            # 5. 結果サマリー生成
            summary = self._generate_result_summary(compliance_report, architecture_result, quality_result)

            result = QualityGateResult(
                mode=self.mode,
                passed=quality_result["passed"],
                exit_code=quality_result["exit_code"],
                compliance_percentage=compliance_report.compliance_percentage,
                violations_count=len(compliance_report.violations),
                critical_violations=quality_result["critical_violations"],
                warnings_count=quality_result["warnings_count"],
                report_path=str(report_path) if report_path else None,
                summary=summary,
            )

            # 6. 結果出力
            await self._output_results(result)

            return result

        except Exception as e:
            self.logger.exception("品質ゲート実行エラー")
            return QualityGateResult(
                mode=self.mode,
                passed=False,
                exit_code=ExitCode.ERROR,
                compliance_percentage=0.0,
                violations_count=0,
                critical_violations=0,
                warnings_count=0,
                report_path=None,
                summary={"error": str(e)},
            )

    def _evaluate_quality(self, compliance_report: object, architecture_result: object) -> dict[str, Any]:
        """品質評価

        Args:
            compliance_report: 準拠性レポート
            architecture_result: アーキテクチャ分析結果

        Returns:
            品質評価結果
        """
        standards = self.quality_standards[self.mode]

        # 違反数集計
        critical_violations = sum(1 for v in compliance_report.violations if v.severity == ViolationSeverity.CRITICAL)

        high_violations = sum(1 for v in compliance_report.violations if v.severity == ViolationSeverity.HIGH)

        total_violations = len(compliance_report.violations)

        # 品質基準チェック
        checks = {
            "compliance_percentage": compliance_report.compliance_percentage >= standards["min_compliance_percentage"],
            "critical_violations": critical_violations <= standards["max_critical_violations"],
            "high_violations": high_violations <= standards["max_high_violations"],
            "total_violations": total_violations <= standards["max_total_violations"],
            "architecture_health": architecture_result.architecture_health_score
            >= standards["require_architecture_health"],
        }

        # 全体合否判定
        passed = all(checks.values()) if self.mode != QualityGateMode.REPORT_ONLY else True

        # 終了コード決定
        if not passed:
            exit_code = ExitCode.VIOLATIONS if critical_violations > 0 else ExitCode.WARNINGS
        else:
            exit_code = ExitCode.SUCCESS

        # 警告数計算
        warnings_count = sum(
            1 for v in compliance_report.violations if v.severity in [ViolationSeverity.MEDIUM, ViolationSeverity.LOW]
        )

        return {
            "passed": passed,
            "exit_code": exit_code,
            "checks": checks,
            "critical_violations": critical_violations,
            "high_violations": high_violations,
            "total_violations": total_violations,
            "warnings_count": warnings_count,
            "standards": standards,
        }

    async def _generate_quality_report(self, compliance_report: object, architecture_result: object, quality_result: object) -> Path | None:
        """品質レポート生成

        Args:
            compliance_report: 準拠性レポート
            architecture_result: アーキテクチャ分析結果
            quality_result: 品質評価結果

        Returns:
            レポートファイルパス
        """
        # レポート出力ディレクトリ
        reports_dir = self.project_root / "reports" / "ddd_quality"
        reports_dir.mkdir(parents=True, exist_ok=True)

        # CI環境では詳細レポートを生成
        if self.is_ci_environment or self.mode == QualityGateMode.CI:
            report_path = reports_dir / f"ddd_quality_gate_{self.mode.value}.json"

            report_data: dict[str, Any] = {
                "timestamp": compliance_report.timestamp.isoformat(),
                "mode": self.mode.value,
                "project_root": str(self.project_root),
                "quality_gate": {
                    "passed": quality_result["passed"],
                    "exit_code": quality_result["exit_code"].value,
                    "standards": quality_result["standards"],
                    "checks": quality_result["checks"],
                },
                "compliance": {
                    "percentage": compliance_report.compliance_percentage,
                    "total_files": compliance_report.total_files_analyzed,
                    "violations": [
                        {
                            "file_path": v.file_path,
                            "line_number": v.line_number,
                            "type": v.violation_type,
                            "severity": v.severity.value,
                            "description": v.description,
                            "recommendation": v.recommendation,
                        }
                        for v in compliance_report.violations
                    ],
                    "layer_compliance": compliance_report.layer_compliance,
                },
                "architecture": {
                    "health_score": architecture_result.architecture_health_score,
                    "violations": [
                        {
                            "source_layer": v.source_layer,
                            "target_layer": v.target_layer,
                            "type": v.violation_type.value,
                            "severity": v.severity,
                            "count": v.count,
                            "recommendation": v.recommendation,
                        }
                        for v in architecture_result.layer_violations
                    ],
                    "circular_dependencies": architecture_result.circular_dependencies,
                    "layer_metrics": architecture_result.layer_metrics,
                },
            }

            # UnifiedFileStorageServiceを使用してレポートを保存
            storage_service = UnifiedFileStorageService()
            storage_service.save(
                file_path=report_path,
                content=report_data,
                content_type=FileContentType.API_RESPONSE,
                metadata={
                    "report_type": "ddd_quality_gate",
                    "mode": self.mode.value,
                    "passed": quality_result["passed"],
                    "compliance_percentage": compliance_report.compliance_percentage,
                },
            )

            return report_path

        return None

    def _generate_result_summary(self, compliance_report: object, architecture_result: object, quality_result: object) -> dict[str, Any]:
        """結果サマリー生成

        Args:
            compliance_report: 準拠性レポート
            architecture_result: アーキテクチャ分析結果
            quality_result: 品質評価結果

        Returns:
            結果サマリー
        """
        return {
            "quality_gate_passed": quality_result["passed"],
            "compliance_percentage": compliance_report.compliance_percentage,
            "architecture_health_score": architecture_result.architecture_health_score,
            "total_violations": quality_result["total_violations"],
            "critical_violations": quality_result["critical_violations"],
            "high_violations": quality_result["high_violations"],
            "warnings": quality_result["warnings_count"],
            "circular_dependencies": len(architecture_result.circular_dependencies),
            "standards_met": quality_result["checks"],
            "recommendations": compliance_report.summary.get("recommendations", []),
        }

    async def _output_results(self, result: QualityGateResult) -> None:
        """結果出力

        Args:
            result: 品質ゲート結果
        """
        # コンソール出力（CI環境では簡潔に）
        if self.is_ci_environment:
            await self._output_ci_results(result)
        else:
            await self._output_interactive_results(result)

        # ログ出力
        if result.passed:
            self.logger.info(
                "DDD品質ゲート合格 - 準拠率: %.1f%%",
                result.compliance_percentage,
            )
        else:
            self.logger.warning(
                "DDD品質ゲート不合格 - 準拠率: %.1f%%, クリティカル違反: %s件",
                result.compliance_percentage,
                result.critical_violations,
            )

    async def _output_ci_results(self, result: QualityGateResult) -> None:
        """CI環境用結果出力"""
        status = "✅ PASSED" if result.passed else "❌ FAILED"

        self.console_service.print(f"DDD Quality Gate: {status}")
        self.console_service.print(f"Compliance: {result.compliance_percentage:.1f}%")
        self.console_service.print(f"Violations: {result.violations_count} (Critical: {result.critical_violations})")
        self.console_service.print(f"Architecture Health: {result.summary.get('architecture_health_score', 0):.2f}")

        if result.report_path:
            self.console_service.print(f"Report: {result.report_path}")

        if not result.passed:
            self.console_service.print("\nFailing Checks:")
            for check, passed in result.summary.get("standards_met", {}).items():
                if not passed:
                    self.console_service.print(f"  ❌ {check}")

    async def _output_interactive_results(self, result: QualityGateResult) -> None:
        """インタラクティブ環境用結果出力"""
        # Rich ConsoleでカラフルなLive表示
        # DDD準拠: Infrastructure→Presentation依存を除去
        # from noveler.presentation.shared.shared_utilities import console

        status_color = "green" if result.passed else "red"
        status_text = "合格" if result.passed else "不合格"

        console.print(f"\n[{status_color}]🎯 DDD品質ゲート結果: {status_text}[/{status_color}]")
        console.print(f"📊 DDD準拠率: {result.compliance_percentage:.1f}%")
        console.print(
            f"🏗️  アーキテクチャ健全性: {result.summary.get('architecture_health_score', 0):.2f}"
        )

        # 違反統計
        if result.violations_count > 0:
            console.print("\n📋 違反統計:")
            console.print(f"  🔴 クリティカル: {result.critical_violations}件")
            console.print(f"  🟠 高重要度: {result.summary.get('high_violations', 0)}件")
            console.print(f"  🟡 警告: {result.warnings_count}件")
            console.print(f"  📊 合計: {result.violations_count}件")

        # 循環依存
        circular_deps = result.summary.get("circular_dependencies", 0)
        if circular_deps > 0:
            console.print(f"🔄 循環依存: {circular_deps}件")

        # 品質基準チェック結果
        console.print("\n🎚️  品質基準:")
        for check, passed in result.summary.get("standards_met", {}).items():
            status_icon = "✅" if passed else "❌"
            console.print(f"  {status_icon} {check}")

        # 推奨事項
        recommendations = result.summary.get("recommendations", [])
        if recommendations:
            console.print("\n💡 推奨事項:")
            for i, rec in enumerate(recommendations[:5], 1):
                console.print(f"  {i}. {rec}")

        if result.report_path:
            console.print(f"\n📄 詳細レポート: {result.report_path}")


async def main() -> None:
    """メイン関数（CLI実行用）"""
    parser = argparse.ArgumentParser(description="DDD品質ゲート")

    parser.add_argument(
        "--mode",
        choices=[mode.value for mode in QualityGateMode],
        default=QualityGateMode.MODERATE.value,
        help="品質ゲートモード",
    )

    parser.add_argument(
        "--check",
        choices=["all", "compliance", "dependencies", "violations"],
        default="all",
        help="実行するチェック種類",
    )

    parser.add_argument("--project-root", type=Path, default=Path.cwd(), help="プロジェクトルートパス")

    parser.add_argument("--report", action="store_true", help="詳細レポート出力")

    parser.add_argument("--output", type=Path, help="出力ファイルパス")

    args = parser.parse_args()

    # モード設定
    mode = QualityGateMode(args.mode)

    # 品質ゲート実行
    quality_gate = DDDQualityGate(args.project_root, mode)
    result = await quality_gate.run_quality_gate()

    # 追加レポート出力
    if args.report and args.output:
        report_data: dict[str, Any] = {
            "timestamp": result.summary.get("timestamp", ""),
            "mode": result.mode.value,
            "passed": result.passed,
            "summary": result.summary,
        }

        # UnifiedFileStorageServiceを使用して追加レポートを保存
        storage_service = UnifiedFileStorageService()
        storage_service.save(
            file_path=args.output,
            content=report_data,
            content_type=FileContentType.API_RESPONSE,
            metadata={"report_type": "ddd_quality_gate_additional", "mode": result.mode.value, "passed": result.passed},
        )

    # 終了コード設定
    sys.exit(result.exit_code.value)


if __name__ == "__main__":
    asyncio.run(main())



#!/usr/bin/env python3
"""段階的品質チェックMCPツールの統合テスト

MCPサーバーの段階的品質チェック機能をエンドツーエンドでテスト。
B20準拠: 実際のMCPサーバー経由での段階的実行を検証。

仕様書: SPEC-MCP-PROGRESSIVE-CHECK-001
"""

import json
from pathlib import Path
from unittest.mock import AsyncMock, Mock, patch

import pytest
import src.mcp_servers.noveler.main as mcp_main

from noveler.presentation.mcp import dispatcher as mcp_dispatcher

# MCPサーバーのインポートテスト
try:
    from src.mcp_servers.noveler.main import server as mcp_server
    MCP_SERVER_AVAILABLE = True
except ImportError:
    MCP_SERVER_AVAILABLE = False
    mcp_server = None


class TestProgressiveCheckMCPTools:
    """段階的品質チェックMCPツールのテスト"""

    @pytest.fixture
    async def mcp_server_instance(self):
        """テスト用MCPサーバー"""
        if not MCP_SERVER_AVAILABLE:
            pytest.skip("MCPサーバーが利用できません")
        return mcp_server

    @pytest.fixture
    def temp_project(self, tmp_path_factory):
        """テスト用プロジェクト環境"""
        project_root = Path(tmp_path_factory.mktemp("progressive_project"))

        # プロジェクト構造作成
        (project_root / "manuscripts").mkdir()
        (project_root / ".noveler" / "checks").mkdir(parents=True)

        # テスト原稿作成
        test_manuscript = project_root / "manuscripts" / "episode_001.md"
        test_manuscript.write_text(
            "これはテスト用の原稿です。誤字や文法の問題が含まれているかもしれません。",
            encoding="utf-8"
        )

        return project_root

    @pytest.mark.asyncio
    async def test_get_check_tasks_tool(self, mcp_server_instance, temp_project):
        """get_check_tasksツールのテスト"""

        session_id = "EP001_202509270900"
        tasks_payload = {
            "session_id": session_id,
            "episode_number": 1,
            "current_step": 1,
            "current_task": {
                "id": 1,
                "name": "誤字脱字チェック",
                "phase": "basic_quality",
                "description": "基本的な誤字脱字の検出",
            },
            "executable_tasks": [
                {
                    "id": 1,
                    "name": "誤字脱字チェック",
                    "phase": "basic_quality",
                    "description": "基本的な誤字脱字の検出",
                    "estimated_duration": "3-5分",
                },
                {
                    "id": 2,
                    "name": "文法・表記統一チェック",
                    "phase": "basic_quality",
                    "description": "文法の正確性と表記の一貫性を確認",
                    "estimated_duration": "5-8分",
                },
            ],
            "progress": {"completed": 0, "total": 12, "percentage": 0.0},
            "llm_instruction": "LLMにタスクリストを提示してください",
            "next_action": "execute_check_step",
            "phase_info": {"phase": "basic_quality"},
        }

        with patch("noveler.domain.services.progressive_check_manager.ProgressiveCheckManager") as mock_manager:
            mock_instance = Mock()
            mock_manager.return_value = mock_instance
            mock_instance.get_check_tasks.return_value = tasks_payload

            result = await mcp_dispatcher.dispatch(
                "get_check_tasks",
                {
                    "episode_number": 1,
                    "project_root": str(temp_project),
                },
            )

        mock_instance.get_check_tasks.assert_called_once()
        assert result["success"] is True
        assert result["session_id"] == session_id
        assert result["execution_method"] == "progressive_check_manager"
        assert result["tasks_info"]["session_id"] == session_id
        assert len(result["tasks_info"]["executable_tasks"]) == 2

    @pytest.mark.asyncio
    async def test_execute_check_step_tool(self, mcp_server_instance, temp_project):
        """execute_check_stepツールのテスト"""
        with patch("noveler.domain.services.progressive_check_manager.ProgressiveCheckManager") as mock_manager:
            mock_instance = Mock()
            mock_manager.return_value = mock_instance
            mock_instance.session_id = "EP001_202509270901"
            mock_instance.execute_check_step.return_value = {
                "step_id": 1,
                "success": True,
                "execution_time": 3.2,
                "issues_found": 2,
                "quality_score": 88.5,
                "corrections": [
                    "1行目: 「かもしれません」→「かもしれません」（助詞の統一）",
                    "1行目: 「誤字」→「誤字」（表記確認完了）",
                ],
                "next_step": 2,
                "files": {
                    "input_file": "EP001_202501091430_step_1_input.json",
                    "output_file": "EP001_202501091430_step_1_output.json",
                },
            }

            result = await mcp_dispatcher.dispatch(
                "execute_check_step",
                {
                    "episode_number": 1,
                    "step_id": 1,
                    "input_data": {
                        "manuscript_content": "テスト原稿内容",
                        "check_focus": "typo_detection",
                    },
                    "project_root": str(temp_project),
                },
            )

        mock_instance.execute_check_step.assert_called_once_with(1, {
            "manuscript_content": "テスト原稿内容",
            "check_focus": "typo_detection",
        }, False)
        assert result["success"] is True
        assert result["execution_result"]["step_id"] == 1
        assert result["execution_result"]["session_id"] == "EP001_202509270901"
        assert result["execution_result"]["next_step"] == 2
        assert len(result["execution_result"]["corrections"]) == 2

    @pytest.mark.asyncio
    async def test_execute_check_step_command_returns_session_info(self, temp_project):
        """execute_check_stepコマンドのレスポンスにsession情報が含まれるかを確認"""

        with patch("noveler.domain.services.progressive_check_manager.ProgressiveCheckManager") as mock_manager:
            mock_instance = Mock()
            mock_manager.return_value = mock_instance
            mock_instance.execute_check_step.return_value = {
                "success": False,
                "session_id": "QC_EP001_TEST",
                "error": "Session corrupted",
                "session_reset_required": True,
            }

            result = await mcp_dispatcher.dispatch(
                "execute_check_step",
                {
                    "episode_number": 1,
                    "step_id": 5,
                    "input_data": {},
                    "project_root": str(temp_project),
                },
            )

        assert result["execution_method"] == "progressive_check_manager"
        assert result["execution_result"].get("session_id") == "QC_EP001_TEST"
        assert result["execution_result"].get("session_reset_required") is not None
        assert result["success"] is False

    @pytest.mark.asyncio
    async def test_get_check_status_tool(self, temp_project):
        """get_check_statusツールのテスト"""
        with patch("noveler.domain.services.progressive_check_manager.ProgressiveCheckManager") as mock_manager:
            mock_instance = Mock()
            mock_manager.return_value = mock_instance

            # ステータス情報のモック
            mock_instance.get_check_status.return_value = {
                "session_id": "EP001_202501091430",
                "episode_number": 1,
                "total_steps": 12,
                "completed_steps": 3,
                "progress_percentage": 25.0,
                "current_phase": "basic_quality",
                "last_completed_step": 3,
                "next_step": 4,
                "estimated_remaining_time": "45-60分",
                "session_start_time": "2025-01-09T14:30:00+09:00"
            }
            mock_instance.session_id = "EP001_202501091430"

            result = await mcp_dispatcher.dispatch(
                "get_check_status",
                {
                    "episode_number": 1,
                    "project_root": str(temp_project),
                },
            )

        mock_instance.get_check_status.assert_called_once()
        assert result["success"] is True
        assert result["session_id"] == "EP001_202501091430"
        status = result["status_info"]
        assert status["episode_number"] == 1
        assert status["total_steps"] == 12
        assert status["session_id"] == "EP001_202501091430"
        assert status["completed_steps"] == 3
        assert status["progress_percentage"] == 25.0
        assert status["current_phase"] == "basic_quality"
        assert status["next_step"] == 4
        assert result["session_id"].startswith("EP001_")

    @pytest.mark.asyncio
    async def test_check_basic_failure_contains_error_log(self, temp_project):
        """check_basicがCLI失敗時にerror_logを返却するか確認"""

        failure_payload = {
            "success": False,
            "error": "CLI execution failed",
            "error_log": "Traceback: simulated error",
        }

        with patch(
            "src.mcp_servers.noveler.main.execute_novel_command",
            new=AsyncMock(return_value=failure_payload),
        ) as mock_execute:
            response = await mcp_main.call_tool(
                "check_basic",
                {"episode_number": 1, "project_root": str(temp_project)},
            )

        mock_execute.assert_awaited_once()
        assert response, "call_tool should return a response"
        payload = json.loads(response[0].text)
        assert payload["success"] is False
        assert payload["error_log"] == "Traceback: simulated error"
        assert payload["error"] == "CLI execution failed"

    @pytest.mark.asyncio
    async def test_get_check_history_tool(self, temp_project):
        """get_check_historyツールのテスト"""
        # Arrange
        episode_number = 1

        with patch("noveler.domain.services.progressive_check_manager.ProgressiveCheckManager") as mock_manager:
            mock_instance = Mock()
            mock_manager.return_value = mock_instance

            # 履歴データのモック
            mock_instance.get_execution_history.return_value = [
                {
                    "step_id": 1,
                    "step_name": "誤字脱字チェック",
                    "executed_at": "2025-01-09T14:31:00+09:00",
                    "execution_time": 3.2,
                    "success": True,
                    "issues_found": 2,
                    "quality_score": 88.5
                },
                {
                    "step_id": 2,
                    "step_name": "文法・表記統一チェック",
                    "executed_at": "2025-01-09T14:35:00+09:00",
                    "execution_time": 4.1,
                    "success": True,
                    "issues_found": 1,
                    "quality_score": 92.0
                },
                {
                    "step_id": 3,
                    "step_name": "読みやすさ基礎チェック",
                    "executed_at": "2025-01-09T14:40:00+09:00",
                    "execution_time": 5.8,
                    "success": True,
                    "issues_found": 0,
                    "quality_score": 95.0
                }
            ]

            # Act: 履歴取得
            result = mock_instance.get_execution_history()

        # Assert
        assert len(result) == 3

        # 各履歴項目の検証
        for i, record in enumerate(result, 1):
            assert record["step_id"] == i
            assert record["success"] is True
            assert record["execution_time"] > 0
            assert record["quality_score"] > 80
            assert "step_name" in record
            assert "executed_at" in record

    @pytest.mark.asyncio
    async def test_step_by_step_workflow(self, temp_project):
        """ステップバイステップワークフローの統合テスト"""
        # Arrange
        episode_number = 1

        with patch("noveler.domain.services.progressive_check_manager.ProgressiveCheckManager") as mock_manager:
            mock_instance = Mock()
            mock_manager.return_value = mock_instance

            # 段階的実行のシミュレーション
            execution_sequence = []

            # ステップ1: 誤字脱字チェック
            mock_instance.execute_check_step.side_effect = [
                {
                    "step_id": 1,
                    "success": True,
                    "execution_time": 3.2,
                    "issues_found": 2,
                    "quality_score": 88.5,
                    "next_step": 2,
                    "phase": "basic_quality"
                },
                {
                    "step_id": 2,
                    "success": True,
                    "execution_time": 4.1,
                    "issues_found": 1,
                    "quality_score": 92.0,
                    "next_step": 3,
                    "phase": "basic_quality"
                },
                {
                    "step_id": 3,
                    "success": True,
                    "execution_time": 5.8,
                    "issues_found": 0,
                    "quality_score": 95.0,
                    "next_step": 4,
                    "phase": "story_quality"  # 次のフェーズに進む
                }
            ]

            # Act: 基本品質フェーズ（ステップ1-3）を段階的に実行
            results = []
            for step_id in range(1, 4):
                result = mock_instance.execute_check_step(step_id, {
                    "episode_number": episode_number,
                    "step_focus": f"step_{step_id}"
                })
                results.append(result)
                execution_sequence.append(step_id)

        # Assert
        assert len(results) == 3
        assert len(execution_sequence) == 3

        # 各ステップの成功を確認
        for i, result in enumerate(results, 1):
            assert result["step_id"] == i
            assert result["success"] is True
            assert result["quality_score"] > 80

        # 段階的な品質向上を確認
        scores = [r["quality_score"] for r in results]
        assert scores[0] < scores[1] < scores[2]  # 品質スコアが段階的に向上

        # フェーズ遷移の確認
        assert results[0]["phase"] == "basic_quality"
        assert results[1]["phase"] == "basic_quality"
        assert results[2]["phase"] == "story_quality"  # フェーズ変更

    @pytest.mark.asyncio
    async def test_file_io_integration(self, temp_project):
        """ファイル入出力統合テスト"""
        # Arrange
        episode_number = 1

        with patch("noveler.domain.services.progressive_check_manager.ProgressiveCheckManager") as mock_manager:
            mock_instance = Mock()
            mock_manager.return_value = mock_instance

            # ファイルI/Oのモック
            expected_session_id = "EP001_202501091430"
            mock_instance.session_id = expected_session_id

            input_file_path = temp_project / ".noveler" / "checks" / expected_session_id / f"{expected_session_id}_step_1_input.json"
            output_file_path = temp_project / ".noveler" / "checks" / expected_session_id / f"{expected_session_id}_step_1_output.json"

            mock_instance.save_step_input.return_value = input_file_path
            mock_instance.save_step_output.return_value = output_file_path

            # ディレクトリ作成
            input_file_path.parent.mkdir(parents=True, exist_ok=True)

            # テストファイル作成
            input_data = {
                "step_id": 1,
                "episode_number": episode_number,
                "manuscript_content": "テスト原稿",
                "timestamp": "2025-01-09T14:30:00+09:00"
            }

            output_data = {
                "step_id": 1,
                "issues_found": 2,
                "quality_score": 88.5,
                "corrections": ["修正1", "修正2"],
                "execution_time": 3.2
            }

            # ファイル保存のシミュレーション
            input_file_path.write_text(json.dumps(input_data, ensure_ascii=False, indent=2), encoding="utf-8")
            output_file_path.write_text(json.dumps(output_data, ensure_ascii=False, indent=2), encoding="utf-8")

            # Act
            saved_input_file = mock_instance.save_step_input(1, input_data)
            saved_output_file = mock_instance.save_step_output(1, output_data)

        # Assert
        assert saved_input_file == input_file_path
        assert saved_output_file == output_file_path

        # ファイル内容の確認
        assert input_file_path.exists()
        assert output_file_path.exists()

        loaded_input = json.loads(input_file_path.read_text(encoding="utf-8"))
        loaded_output = json.loads(output_file_path.read_text(encoding="utf-8"))

        assert loaded_input == input_data
        assert loaded_output == output_data

    @pytest.mark.asyncio
    async def test_error_recovery_workflow(self, temp_project):
        """エラー復旧ワークフローのテスト"""
        # Arrange
        episode_number = 1

        with patch("noveler.domain.services.progressive_check_manager.ProgressiveCheckManager") as mock_manager:
            mock_instance = Mock()
            mock_manager.return_value = mock_instance

            # エラー発生ケースのモック
            mock_instance.execute_check_step.side_effect = [
                # ステップ1: 成功
                {
                    "step_id": 1,
                    "success": True,
                    "quality_score": 88.5,
                    "next_step": 2
                },
                # ステップ2: エラー発生
                {
                    "step_id": 2,
                    "success": False,
                    "error": "処理中にエラーが発生しました",
                    "error_type": "processing_error",
                    "recoverable": True
                },
                # ステップ2: リトライ成功
                {
                    "step_id": 2,
                    "success": True,
                    "quality_score": 90.0,
                    "next_step": 3,
                    "retry_count": 1
                }
            ]

            # Act: エラー発生とリトライのワークフロー
            results = []

            # ステップ1: 成功
            result1 = mock_instance.execute_check_step(1, {"test": "data"})
            results.append(result1)

            # ステップ2: エラー発生
            result2_error = mock_instance.execute_check_step(2, {"test": "data"})
            results.append(result2_error)

            # ステップ2: リトライ
            if not result2_error["success"] and result2_error.get("recoverable"):
                result2_retry = mock_instance.execute_check_step(2, {"test": "data", "retry": True})
                results.append(result2_retry)

        # Assert
        assert len(results) == 3

        # ステップ1: 成功
        assert results[0]["success"] is True

        # ステップ2: 最初はエラー
        assert results[1]["success"] is False
        assert "error" in results[1]
        assert results[1]["recoverable"] is True

        # ステップ2: リトライ後は成功
        assert results[2]["success"] is True
        assert results[2]["retry_count"] == 1
        assert results[2]["quality_score"] > 0

    @pytest.mark.asyncio
    async def test_backward_compatibility(self, temp_project):
        """後方互換性テスト（従来の一括実行との併用）"""
        # Arrange
        with patch("src.mcp_servers.noveler.json_conversion_server.JSONConversionServer._execute_progressive_check") as mock_progressive:
            # 段階的実行ガイダンスのモック
            mock_progressive.return_value = """
🎯 段階的品質チェック機能が利用可能です
新しい段階的チェックシステムの使用方法:
1. get_check_tasks(episode_number=1)
2. execute_check_step(episode_number=1, step_id=1)
💡 段階的指導で品質向上
"""

            # Act: 既存checkツールでprogressive=Trueを使用
            # これは実際のMCPサーバーのjson_conversion_server.pyのcheckツールをシミュレート
            guidance_result = mock_progressive(1, "all", str(temp_project))

        # Assert
        assert "段階的品質チェック機能が利用可能です" in guidance_result
        assert "get_check_tasks" in guidance_result
        assert "execute_check_step" in guidance_result
        assert "段階的指導で品質向上" in guidance_result


@pytest.mark.e2e
class TestProgressiveCheckE2E:
    """段階的品質チェックのエンドツーエンドテスト"""

    @pytest.mark.asyncio
    async def test_complete_12_step_workflow(self, temp_project_dir):
        """完全な12ステップワークフローのテスト（簡略版）"""
        # Arrange
        with patch("noveler.domain.services.progressive_check_manager.ProgressiveCheckManager") as mock_manager:
            mock_instance = Mock()
            mock_manager.return_value = mock_instance

            # 12ステップ全体の実行結果をモック
            all_steps_results = []
            phases = [
                ("basic_quality", [1, 2, 3]),
                ("story_quality", [4, 5, 6]),
                ("structure_quality", [7, 8, 9]),
                ("expression_quality", [10, 11, 12])
            ]

            step_results = []
            for phase_name, step_ids in phases:
                for step_id in step_ids:
                    step_results.append({
                        "step_id": step_id,
                        "success": True,
                        "phase": phase_name,
                        "quality_score": 80 + step_id,  # 段階的向上
                        "next_step": step_id + 1 if step_id < 12 else None,
                        "execution_time": 2.0 + step_id * 0.5
                    })

            mock_instance.execute_check_step.side_effect = step_results

            # 最終ステータス
            mock_instance.get_execution_status.return_value = {
                "episode_number": 1,
                "total_steps": 12,
                "completed_steps": 12,
                "progress_percentage": 100.0,
                "current_phase": "expression_quality",
                "final_quality_score": 92.0,
                "session_completed": True
            }

        # Act: 12ステップすべてを実行（簡略版）
        execution_results = []
        for step_id in range(1, 13):
            result = mock_instance.execute_check_step(step_id, {"step_test": step_id})
            execution_results.append(result)

        final_status = mock_instance.get_execution_status()

        # Assert
        assert len(execution_results) == 12

        # 各フェーズの完了確認
        basic_steps = execution_results[0:3]  # ステップ1-3
        story_steps = execution_results[3:6]  # ステップ4-6
        structure_steps = execution_results[6:9]  # ステップ7-9
        expression_steps = execution_results[9:12]  # ステップ10-12

        for steps in [basic_steps, story_steps, structure_steps, expression_steps]:
            for step in steps:
                assert step["success"] is True

        # 品質スコアの段階的向上確認
        scores = [r["quality_score"] for r in execution_results]
        assert all(scores[i] <= scores[i+1] for i in range(len(scores)-1))  # 単調増加

        # 最終ステータス確認
        assert final_status["completed_steps"] == 12
        assert final_status["progress_percentage"] == 100.0
        assert final_status["session_completed"] is True
        assert final_status["final_quality_score"] > 90


if __name__ == "__main__":
    pytest.main([__file__, "-v", "--tb=short"])



#!/usr/bin/env python3

"""Domain.entities.quality_check_session
Where: Domain entity tracking quality check sessions.
What: Stores session configuration, progress, and outcomes.
Why: Supports auditing and reruns of quality checks.
"""

from __future__ import annotations

"""品質チェックセッションエンティティ(DDD実装)

品質チェックのドメインモデル。
レガシーの品質チェッカーをDDDアーキテクチャで再実装。
"""


from dataclasses import dataclass, field
from enum import Enum
from typing import TYPE_CHECKING, Any, ClassVar

from noveler.domain.value_objects.project_time import ProjectTimezone, project_now

if TYPE_CHECKING:
    from datetime import datetime

# ファイルコンテンツの型エイリアス
FileContent = str | bytes | dict[str, Any]

# JSTタイムゾーン
JST = ProjectTimezone.jst().timezone


class CheckType(Enum):
    """チェックタイプ"""

    BASIC_STYLE = "basic_style"
    COMPOSITION = "composition"
    CHARACTER_CONSISTENCY = "character_consistency"
    READABILITY = "readability"
    INVALID_KANJI = "invalid_kanji"
    FORESHADOWING = "foreshadowing"

    # v2.0 執筆品質強化チェックタイプ (SPEC-QUALITY-023 v2.0)
    EMOTION_DEPTH = "emotion_depth"          # 感情表現深度チェック
    ANTAGONIST_PERSONALITY = "antagonist"    # 対立キャラクター個性化チェック
    TENSION_BALANCE = "tension_balance"      # 緊張緩和バランスチェック


class Severity(Enum):
    """問題の重要度"""

    ERROR = "error"
    WARNING = "warning"
    INFO = "info"


class QualityGrade(Enum):
    """品質グレード"""

    S = "S"  # 90以上
    A = "A"  # 80以上
    B = "B"  # 70以上
    C = "C"  # 60以上
    D = "D"  # 60未満

    @property
    def display_name(self) -> str:
        """表示名を取得"""
        names = {
            "S": "S級(秀逸)",
            "A": "A級(優良)",
            "B": "B級(標準)",
            "C": "C級(要改善)",
            "D": "D級(要大幅改善)",
        }
        return names[self.value]

    def __gt__(self, other: QualityGrade) -> bool:
        """グレードの比較(大きい)"""
        if not isinstance(other, QualityGrade):
            return NotImplemented
        order = ["D", "C", "B", "A", "S"]
        return order.index(self.value) > order.index(other.value)

    def __lt__(self, other: QualityGrade) -> bool:
        """グレードの比較(小さい)"""
        if not isinstance(other, QualityGrade):
            return NotImplemented
        order = ["D", "C", "B", "A", "S"]
        return order.index(self.value) < order.index(other.value)


@dataclass(frozen=True)
class QualityScore:
    """品質スコア値オブジェクト(0-100)"""

    value: float

    def __post_init__(self) -> None:
        """バリデーション"""
        if not 0 <= self.value <= 100:
            msg = "スコアは0-100の範囲内である必要があります"
            raise ValueError(msg)

    def __hash__(self) -> int:
        """ハッシュ値を生成"""
        return hash(self.value)

    def to_grade(self) -> QualityGrade:
        """スコアからグレードを判定"""
        if self.value >= 90:
            return QualityGrade.S
        if self.value >= 80:
            return QualityGrade.A
        if self.value >= 70:
            return QualityGrade.B
        if self.value >= 60:
            return QualityGrade.C
        return QualityGrade.D

    def __lt__(self, other: QualityScore) -> bool:
        if not isinstance(other, QualityScore):
            return NotImplemented
        return self.value < other.value

    def __gt__(self, other: object) -> bool:
        if not isinstance(other, QualityScore):
            return NotImplemented
        return self.value > other.value

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, QualityScore):
            return NotImplemented
        return self.value == other.value


@dataclass(frozen=True)
class QualityIssue:
    """品質問題値オブジェクト"""

    type: str
    message: str
    severity: Severity
    line_number: int | None = None
    position: int | None = None
    suggestion: str | None = None
    auto_fixable: bool = False


@dataclass(frozen=True)
class QualityCheckResult:
    """個別チェック結果値オブジェクト"""

    check_type: CheckType
    score: QualityScore
    issues: list[QualityIssue]
    metadata: dict[str, Any] = field(default_factory=dict)
    execution_time: float = 0.0
    suggestions: list[str] = field(default_factory=list)

    @property
    def error_count(self) -> int:
        """エラー数を取得"""
        return sum(1 for issue in self.issues if issue.severity == Severity.ERROR)

    @property
    def warning_count(self) -> int:
        """警告数を取得"""
        return sum(1 for issue in self.issues if issue.severity == Severity.WARNING)

    def to_dict(self) -> dict[str, Any]:
        """辞書形式に変換"""
        return {
            "check_type": self.check_type.value,
            "score": self.score.value,
            "issues": [
                {
                    "type": issue.type,
                    "message": issue.message,
                    "severity": issue.severity.value,
                    "line_number": issue.line_number,
                    "position": issue.position,
                    "suggestion": issue.suggestion,
                    "auto_fixable": issue.auto_fixable,
                }
                for issue in self.issues
            ],
            "metadata": self.metadata,
            "execution_time": self.execution_time,
            "error_count": self.error_count,
            "warning_count": self.warning_count,
            "suggestions": self.suggestions,
        }


class QualityCheckSession:
    """品質チェックセッション(ルートアグリゲート)

    品質チェックの実行単位を表現。
    複数のチェック結果を集約して総合評価を行う。
    """

    # チェッカーの重み設定
    CHECKER_WEIGHTS: ClassVar[dict[CheckType, float]] = {
        CheckType.BASIC_STYLE: 0.25,
        CheckType.COMPOSITION: 0.25,
        CheckType.CHARACTER_CONSISTENCY: 0.20,
        CheckType.READABILITY: 0.20,
        CheckType.INVALID_KANJI: 0.10,
    }

    def __init__(
        self, session_id: str, project_id: str, target_content: str | None = None, config: dict[str, Any] | None = None
    ) -> None:
        """初期化

        Args:
            session_id: セッションID
            project_id: プロジェクトID
            target_content: チェック対象コンテンツ
            config: 設定
        """
        self.session_id = session_id
        self.project_id = project_id
        self.target_content = target_content
        self.config = config or {}
        self.created_at = project_now().datetime
        self.completed_at: datetime | None = None
        self.status = "pending"  # pending, in_progress, completed, failed
        self.check_results: list[QualityCheckResult] = []
        self._total_score_cache: QualityScore | None = None
        self._grade_cache: QualityGrade | None = None

    def add_check_result(self, result: QualityCheckResult) -> None:
        """チェック結果を追加

        Args:
            result: チェック結果

        Raises:
            ValueError: 完了したセッションに追加しようとした場合
        """
        if self.status == "completed":
            msg = "完了したセッションには結果を追加できません"
            raise ValueError(msg)

        # 同じタイプの結果は上書き
        self.check_results = [r for r in self.check_results if r.check_type != result.check_type]
        self.check_results.append(result)

        # キャッシュをクリア
        self._total_score_cache = None
        self._grade_cache = None

        # ステータス更新
        if self.status == "pending":
            self.status = "in_progress"

    def has_check_type(self, check_type: CheckType) -> bool:
        """指定したチェックタイプの結果が存在するか"""
        return any(r.check_type == check_type for r in self.check_results)

    def get_check_result(self, check_type: CheckType) -> QualityCheckResult | None:
        """指定したチェックタイプの結果を取得"""
        for result in self.check_results:
            if result.check_type == check_type:
                return result
        return None

    def calculate_total_score(self) -> QualityScore:
        """総合スコアを計算

        Returns:
            総合スコア(重み付け平均)
        """
        if self._total_score_cache is not None:
            return self._total_score_cache

        if not self.check_results:
            return QualityScore(0.0)

        # 重み付け平均を計算
        total_weight = 0.0
        weighted_sum = 0.0

        for result in self.check_results:
            weight = self.CHECKER_WEIGHTS.get(result.check_type, 0.0)
            weighted_sum += result.score.value * weight
            total_weight += weight

        if total_weight == 0:
            # 重みが設定されていないチェックのみの場合は単純平均
            avg_score = sum(r.score.value for r in self.check_results) / len(self.check_results)
            self._total_score_cache = QualityScore(avg_score)
        else:
            self._total_score_cache = QualityScore(weighted_sum / total_weight)

        return self._total_score_cache

    def determine_grade(self) -> QualityGrade:
        """品質グレードを判定

        Returns:
            品質グレード
        """
        if self._grade_cache is not None:
            return self._grade_cache

        total_score = self.calculate_total_score()
        self._grade_cache = total_score.to_grade()
        return self._grade_cache

    def get_all_issues(self) -> list[QualityIssue]:
        """全ての問題を取得

        Returns:
            全チェック結果の問題リスト
        """
        issues = []
        for result in self.check_results:
            issues.extend(result.issues)
        return issues

    def get_issues_by_severity(self, severity: str) -> list[QualityIssue]:
        """重要度別に問題を取得

        Args:
            severity: 重要度

        Returns:
            指定した重要度の問題リスト
        """
        return [issue for issue in self.get_all_issues() if issue.severity == severity]

    def complete(self) -> None:
        """セッションを完了"""
        self.status = "completed"
        self.completed_at = project_now().datetime

    def fail(self, error_message: str) -> None:
        """セッションを失敗として記録"""
        self.status = "failed"
        self.completed_at = project_now().datetime
        self.config["error_message"] = error_message

    def export_summary(self) -> dict[str, Any]:
        """サマリーをエクスポート

        Returns:
            セッションのサマリー情報
        """
        total_score = self.calculate_total_score()
        grade = self.determine_grade()
        all_issues = self.get_all_issues()

        return {
            "session_id": self.session_id,
            "project_id": self.project_id,
            "status": self.status,
            "created_at": self.created_at.isoformat(),
            "completed_at": self.completed_at.isoformat() if self.completed_at else None,
            "target_file": self.target_content.filepath,
            "total_score": total_score.value,
            "grade": grade.value,
            "grade_display": grade.display_name,
            "check_results": [result.to_dict() for result in self.check_results],
            "total_issues": len(all_issues),
            "error_count": len(self.get_issues_by_severity(Severity.ERROR)),
            "warning_count": len(self.get_issues_by_severity(Severity.WARNING)),
            "info_count": len(self.get_issues_by_severity(Severity.INFO)),
        }

    def get_improvement_suggestions(self) -> list[str]:
        """改善提案を生成

        Returns:
            改善提案のリスト
        """
        suggestions = []

        # スコアが低いチェック項目を特定
        low_score_results = [r for r in self.check_results if r.score.value < 70]

        for result in sorted(low_score_results, key=lambda r: r.score.value):
            check_name = self._get_check_display_name(result.check_type)
            suggestions.append(f"{check_name}のスコアが{result.score.value}点と低めです")

        # 自動修正可能な問題を提案
        auto_fixable_issues = [issue for issue in self.get_all_issues() if issue.auto_fixable]
        if auto_fixable_issues:
            suggestions.append(
                f"{len(auto_fixable_issues)}件の問題は自動修正が可能です。"
                "品質チェックツールの自動修正機能を使用することをお勧めします。"
            )

        return suggestions

    def _get_check_display_name(self, check_type: CheckType) -> str:
        """チェックタイプの表示名を取得"""
        names = {
            CheckType.BASIC_STYLE: "基礎文章作法",
            CheckType.COMPOSITION: "文章構成",
            CheckType.CHARACTER_CONSISTENCY: "キャラクター一貫性",
            CheckType.READABILITY: "読みやすさ",
            CheckType.INVALID_KANJI: "無効漢字",
        }
        return names.get(check_type, check_type.value)



#!/usr/bin/env python3
"""JSON変換システム ヘルスチェック"""


from dataclasses import asdict, dataclass
from pathlib import Path
from typing import Any

from noveler.domain.value_objects.project_time import project_now
from noveler.infrastructure.json.converters.cli_response_converter import CLIResponseConverter
from noveler.infrastructure.json.file_managers.file_reference_manager import FileReferenceManager
from noveler.infrastructure.json.models.response_models import StandardResponseModel


@dataclass
class HealthCheckResult:
    """ヘルスチェック結果"""

    component: str
    status: str  # "healthy", "warning", "critical"
    message: str
    details: dict[str, Any]
    checked_at: str
    response_time_ms: float = 0.0


class JSONSystemHealthChecker:
    """JSON変換システム ヘルスチェッカー"""

    def __init__(self, output_dir: Path | None = None) -> None:
        self.output_dir = output_dir or Path.cwd() / "temp" / "json_output"
        # logger_service経由で注入

    def run_comprehensive_check(self) -> dict[str, Any]:
        """包括的ヘルスチェック実行"""
        start_time = project_now().datetime

        results = []

        # 各コンポーネント チェック実行（例外安全）
        check_methods = [
            ("file_system", self._check_file_system_health),
            ("converter", self._check_converter_health),
            ("file_manager", self._check_file_manager_health),
            ("model_validation", self._check_model_validation_health),
            ("performance", self._check_performance_metrics),
        ]

        for component_name, check_method in check_methods:
            try:
                result = check_method()
                results.append(result)
            except Exception as e:
                # チェックメソッドで例外が発生した場合のフォールバック
                self.logger.exception("ヘルスチェック例外 [%s]", component_name)
                error_result = HealthCheckResult(
                    component=component_name,
                    status="critical",
                    message=f"チェック実行中に例外が発生: {e!s}",
                    details={"error": str(e), "error_type": type(e).__name__},
                    checked_at=project_now().datetime.isoformat(),
                    response_time_ms=0.0,
                )
                results.append(error_result)

        # 総合ステータス判定
        overall_status = self._determine_overall_status(results)

        total_time = (project_now().datetime - start_time).total_seconds() * 1000

        return {
            "overall_status": overall_status,
            "total_check_time_ms": total_time,
            "checked_at": start_time.isoformat(),
            "components": [asdict(result) for result in results],
            "summary": self._generate_summary(results),
        }

    def _check_file_system_health(self) -> HealthCheckResult:
        """ファイルシステム ヘルスチェック"""
        start_time = project_now().datetime

        try:
            # ディレクトリ存在確認
            if not self.output_dir.exists():
                self.output_dir.mkdir(parents=True, exist_ok=True)

            # 書き込み権限確認
            test_file = self.output_dir / ".health_check_test"
            test_file.write_text("health check test")
            test_file.unlink()

            # ディスク容量チェック
            self.output_dir.stat()

            response_time = (project_now().datetime - start_time).total_seconds() * 1000

            return HealthCheckResult(
                component="file_system",
                status="healthy",
                message="ファイルシステム正常",
                details={"output_dir": str(self.output_dir), "writable": True, "disk_available": True},
                checked_at=project_now().datetime.isoformat(),
                response_time_ms=response_time,
            )

        except Exception as e:
            response_time = (project_now().datetime - start_time).total_seconds() * 1000
            return HealthCheckResult(
                component="file_system",
                status="critical",
                message=f"ファイルシステムエラー: {e!s}",
                details={"error": str(e)},
                checked_at=project_now().datetime.isoformat(),
                response_time_ms=response_time,
            )

    def _check_converter_health(self) -> HealthCheckResult:
        """コンバーター ヘルスチェック"""
        start_time = project_now().datetime

        try:
            converter = CLIResponseConverter(output_dir=self.output_dir)

            # テスト変換実行（許可されたコマンドプレフィックスを使用）
            test_cli_result = {
                "success": True,
                "command": "novel health_check",  # 許可されたプレフィックス
                "content": "# ヘルスチェック\n\nテスト内容",
                "execution_time_ms": 100.0,
            }

            json_result = converter.convert(test_cli_result)

            # 結果検証
            if not json_result.get("success", False):
                msg = "変換結果が成功ステータスではありません"
                raise ValueError(msg)

            if "outputs" not in json_result:
                msg = "出力情報がありません"
                raise ValueError(msg)

            response_time = (project_now().datetime - start_time).total_seconds() * 1000

            return HealthCheckResult(
                component="converter",
                status="healthy",
                message="コンバーター正常動作",
                details={"test_conversion": "success", "output_files": json_result["outputs"]["total_files"]},
                checked_at=project_now().datetime.isoformat(),
                response_time_ms=response_time,
            )

        except Exception as e:
            response_time = (project_now().datetime - start_time).total_seconds() * 1000
            return HealthCheckResult(
                component="converter",
                status="critical",
                message=f"コンバーターエラー: {e!s}",
                details={"error": str(e)},
                checked_at=project_now().datetime.isoformat(),
                response_time_ms=response_time,
            )

    def _check_file_manager_health(self) -> HealthCheckResult:
        """ファイルマネージャー ヘルスチェック"""
        start_time = project_now().datetime

        try:
            file_manager = FileReferenceManager(self.output_dir)

            # テストファイル保存・検証
            test_content = "ヘルスチェックテストコンテンツ"
            file_ref = file_manager.save_content(
                content=test_content, content_type="text/plain", filename_prefix="health_check"
            )

            # 完全性検証
            integrity_ok = file_manager.verify_file_integrity(file_ref)
            if not integrity_ok:
                msg = "ファイル完全性チェックに失敗"
                raise ValueError(msg)

            # ファイル読み込みテスト
            loaded_content = file_manager.load_file_content(file_ref)
            if loaded_content != test_content:
                msg = "ファイル内容が一致しません"
                raise ValueError(msg)

            response_time = (project_now().datetime - start_time).total_seconds() * 1000

            return HealthCheckResult(
                component="file_manager",
                status="healthy",
                message="ファイルマネージャー正常動作",
                details={"file_save": "success", "integrity_check": "success", "file_load": "success"},
                checked_at=project_now().datetime.isoformat(),
                response_time_ms=response_time,
            )

        except Exception as e:
            response_time = (project_now().datetime - start_time).total_seconds() * 1000
            return HealthCheckResult(
                component="file_manager",
                status="critical",
                message=f"ファイルマネージャーエラー: {e!s}",
                details={"error": str(e)},
                checked_at=project_now().datetime.isoformat(),
                response_time_ms=response_time,
            )

    def _check_model_validation_health(self) -> HealthCheckResult:
        """モデルバリデーション ヘルスチェック"""
        start_time = project_now().datetime

        try:
            # テストデータでモデル検証（許可されたコマンドを使用）
            test_data = {
                "success": True,
                "command": "novel test_command",  # 許可されたプレフィックス
                "execution_time_ms": 123.45,
                "outputs": {"files": [], "total_files": 0, "total_size_bytes": 0},
                "metadata": {"test": True},
                "created_at": project_now().datetime,
            }

            # StandardResponseModelでバリデーション
            model = StandardResponseModel(**test_data)

            # モデル→辞書変換テスト
            model.model_dump()

            response_time = (project_now().datetime - start_time).total_seconds() * 1000

            return HealthCheckResult(
                component="model_validation",
                status="healthy",
                message="モデルバリデーション正常",
                details={"pydantic_validation": "success", "model_serialization": "success"},
                checked_at=project_now().datetime.isoformat(),
                response_time_ms=response_time,
            )

        except Exception as e:
            response_time = (project_now().datetime - start_time).total_seconds() * 1000
            return HealthCheckResult(
                component="model_validation",
                status="critical",
                message=f"モデルバリデーションエラー: {e!s}",
                details={"error": str(e)},
                checked_at=project_now().datetime.isoformat(),
                response_time_ms=response_time,
            )

    def _check_performance_metrics(self) -> HealthCheckResult:
        """パフォーマンス メトリクス チェック"""
        start_time = project_now().datetime

        try:
            # 複数回テスト実行してパフォーマンス測定
            execution_times = []

            for i in range(5):
                test_start = project_now().datetime

                converter = CLIResponseConverter(output_dir=self.output_dir)
                converter.convert(
                    {
                        "success": True,
                        "command": f"novel performance_test_{i}",  # 許可されたプレフィックス
                        "content": f"パフォーマンステスト {i}",
                    }
                )

                test_time = (project_now().datetime - test_start).total_seconds() * 1000
                execution_times.append(test_time)

            avg_time = sum(execution_times) / len(execution_times)
            max_time = max(execution_times)
            min_time = min(execution_times)

            # パフォーマンス閾値チェック
            status = "healthy"
            if avg_time > 1000:  # 1秒以上
                status = "warning"
            elif avg_time > 2000:  # 2秒以上
                status = "critical"

            response_time = (project_now().datetime - start_time).total_seconds() * 1000

            return HealthCheckResult(
                component="performance",
                status=status,
                message=f"パフォーマンス測定完了: 平均{avg_time:.2f}ms",
                details={
                    "average_time_ms": avg_time,
                    "max_time_ms": max_time,
                    "min_time_ms": min_time,
                    "test_iterations": len(execution_times),
                },
                checked_at=project_now().datetime.isoformat(),
                response_time_ms=response_time,
            )

        except Exception as e:
            response_time = (project_now().datetime - start_time).total_seconds() * 1000
            return HealthCheckResult(
                component="performance",
                status="critical",
                message=f"パフォーマンス測定エラー: {e!s}",
                details={"error": str(e)},
                checked_at=project_now().datetime.isoformat(),
                response_time_ms=response_time,
            )

    def _determine_overall_status(self, results: list[HealthCheckResult]) -> str:
        """総合ステータス判定"""
        statuses = [result.status for result in results]

        if "critical" in statuses:
            return "critical"
        if "warning" in statuses:
            return "warning"
        return "healthy"

    def _generate_summary(self, results: list[HealthCheckResult]) -> dict[str, Any]:
        """サマリー生成"""
        total_components = len(results)
        healthy_count = sum(1 for r in results if r.status == "healthy")
        warning_count = sum(1 for r in results if r.status == "warning")
        critical_count = sum(1 for r in results if r.status == "critical")

        avg_response_time = sum(r.response_time_ms for r in results) / total_components

        return {
            "total_components": total_components,
            "healthy_components": healthy_count,
            "warning_components": warning_count,
            "critical_components": critical_count,
            "health_percentage": (healthy_count / total_components) * 100,
            "average_response_time_ms": avg_response_time,
        }



"""Domain.entities.bulk_quality_check
Where: Domain entity representing bulk quality check runs.
What: Stores batch parameters, per-episode results, and summaries.
Why: Enables aggregated quality insights across multiple episodes.
"""

from __future__ import annotations

"""全話品質チェック エンティティ"""


from dataclasses import dataclass
from typing import TYPE_CHECKING

from noveler.domain.value_objects.project_time import ProjectTimezone, project_now

if TYPE_CHECKING:
    from datetime import datetime


# JSTタイムゾーン
JST = ProjectTimezone.jst().timezone


@dataclass
class BulkQualityCheck:
    """全話品質チェック エンティティ"""

    project_name: str
    episode_range: tuple[int, int] | None = None
    parallel: bool = False
    include_archived: bool = False
    force_recheck: bool = False

    def __post_init__(self) -> None:
        if not self.project_name.strip():
            msg = "Project name cannot be empty"
            raise ValueError(msg)


@dataclass
class QualityRecord:
    """品質記録"""

    episode_number: int
    quality_score: float
    category_scores: dict
    timestamp: datetime


@dataclass
class QualityTrend:
    """品質トレンド"""

    direction: str  # "improving", "stable", "declining"
    slope: float
    confidence: float


class QualityHistory:
    """品質記録履歴"""

    def __init__(self, project_name: str) -> None:
        self.project_name = project_name
        self.records: list[QualityRecord] = []

    def add_record(self, episode_number: int, quality_result: dict) -> None:
        """品質記録を追加"""
        record = QualityRecord(
            episode_number=episode_number,
            quality_score=quality_result.overall_score.to_float(),
            category_scores=quality_result.category_scores.to_dict(),
            timestamp=project_now().datetime,
        )

        self.records.append(record)

    def calculate_trend(self) -> QualityTrend:
        """品質トレンドを計算"""
        if len(self.records) < 2:
            return QualityTrend(direction="stable", slope=0.0, confidence=0.0)

        # 簡単な線形回帰でトレンドを計算
        scores = [record.quality_score for record in self.records]
        x_values = list(range(len(scores)))

        # 傾きを計算
        n = len(scores)
        sum_x = sum(x_values)
        sum_y = sum(scores)
        sum_xy = sum(x * y for x, y in zip(x_values, scores, strict=False))
        sum_x2 = sum(x * x for x in x_values)

        slope = (n * sum_xy - sum_x * sum_y) / (n * sum_x2 - sum_x * sum_x)

        # 方向を判定
        if slope > 1.0:
            direction = "improving"
        elif slope < -1.0:
            direction = "declining"
        else:
            direction = "stable"

        # 信頼度を計算(簡易版)
        confidence = min(abs(slope) / 5.0, 1.0)

        return QualityTrend(direction=direction, slope=slope, confidence=confidence)

    def find_problematic_episodes(self, threshold: float = 70.0) -> list[int]:
        """問題のあるエピソードを特定"""
        return [record.episode_number for record in self.records if record.quality_score < threshold]



"""Domain.services.narrative_depth_services
Where: Domain service module providing reusable business logic.
What: Houses service routines and helpers implemented in this module.
Why: Makes this service behaviour accessible to application workflows.
"""

from __future__ import annotations

"""内面描写深度評価のドメインサービス。

小説の内面描写の深度を分析・評価するためのドメインサービス。
心理描写、感情表現、内面的葛藤などを多層的に評価する。
"""


import re

from noveler.domain.value_objects.narrative_depth_models import (
    DepthLayer,
    DepthPattern,
    LayerScore,
    NarrativeDepthScore,
    TextSegment,
)


class NarrativeDepthAnalyzer:
    """内面描写深度分析のドメインサービス"""

    def __init__(self) -> None:
        self.patterns = self._initialize_patterns()

    def analyze_depth(self, text: str) -> NarrativeDepthScore:
        """テキストの内面描写深度を分析"""
        segments = self._segment_text(text)
        layer_scores = {}

        for layer in DepthLayer:
            score = self._calculate_layer_score(segments, layer)
            layer_scores[layer] = LayerScore(layer=layer, score=score)

        return NarrativeDepthScore(layer_scores=layer_scores)

    def _segment_text(self, text: str) -> list[TextSegment]:
        """テキストを段落単位でセグメント化"""
        paragraphs = text.split("\n\n")
        segments = []
        start_pos = 0

        for paragraph in paragraphs:
            if paragraph.strip():
                end_pos = start_pos + len(paragraph)
                segments.append(
                    TextSegment(
                        content=paragraph.strip(),
                        start_position=start_pos,
                        end_position=end_pos,
                    ),
                )

                start_pos = end_pos + 2  # \n\n分

        return segments

    def _calculate_layer_score(self, segments: list[TextSegment], layer: DepthLayer) -> float:
        """特定の層のスコアを計算"""
        layer_patterns = self._get_patterns_for_layer(layer)
        total_score = 0.0
        total_length = sum(len(segment.content) for segment in segments)

        if total_length == 0:
            return 0.0

        for segment in segments:
            segment_score = self._analyze_segment(segment, layer_patterns)
            # セグメント長による重み付け
            weight = len(segment.content) / total_length
            total_score += segment_score * weight

        # 20点満点にスケーリング
        return min(total_score, 20.0)

    def _get_patterns_for_layer(self, layer: DepthLayer) -> list[DepthPattern]:
        """特定の層のパターンを取得"""
        return [pattern for pattern in self.patterns if pattern.layer == layer]

    def _analyze_segment(self, segment: TextSegment, patterns: list[DepthPattern]) -> float:
        """セグメントの分析"""
        score = 0.0

        for pattern in patterns:
            matches = re.findall(pattern.pattern, segment.content)
            if matches:
                # マッチ数 × 深度レベル × 重み
                pattern_score = len(matches) * pattern.depth_level * pattern.weight
                score += pattern_score

        # セグメント内での密度を考慮
        if len(segment.content) > 0:
            density_factor = min(len(segment.content) / 100, 1.0)
            score *= density_factor

        return score

    def _initialize_patterns(self) -> list[DepthPattern]:
        """評価パターンの初期化"""
        patterns = []

        # 感覚層パターン(五感・身体感覚)
        patterns.extend(
            [
                DepthPattern(r"(震え|痺れ|熱|冷た|重|軽)", DepthLayer.SENSORY, 1),
                DepthPattern(r"(心臓|脈|鼓動|血|汗|涙)", DepthLayer.SENSORY, 2),
                DepthPattern(r"(締め付け|圧迫|解放|緊張|弛緩)", DepthLayer.SENSORY, 2),
                DepthPattern(r"まるで.*?のような(痛み|感覚)", DepthLayer.SENSORY, 3),
            ],
        )

        # 感情層パターン(感情の直接的・間接的表現)
        patterns.extend(
            [
                DepthPattern(r"(嬉しい|悲しい|怖い)", DepthLayer.EMOTIONAL, 1),
                DepthPattern(r"(不安|焦燥|懐かしい|愛しい)", DepthLayer.EMOTIONAL, 2),
                DepthPattern(r"(甘美な|痛いほどの|切ない)", DepthLayer.EMOTIONAL, 3),
                DepthPattern(r"胸が.*?くなる", DepthLayer.EMOTIONAL, 2),
                DepthPattern(r"襲ってくる", DepthLayer.EMOTIONAL, 2),
            ],
        )

        # 思考層パターン(論理的思考・推論・認知プロセス)
        patterns.extend(
            [
                DepthPattern(r"思った|考えた", DepthLayer.COGNITIVE, 1),
                DepthPattern(r"でも.*?違う", DepthLayer.COGNITIVE, 2),
                DepthPattern(r"でも.*?だから", DepthLayer.COGNITIVE, 2),
                DepthPattern(r"いや.*?違う", DepthLayer.COGNITIVE, 2),
                DepthPattern(r"もしかして.*?かも", DepthLayer.COGNITIVE, 2),
                DepthPattern(r"本当は.*?のに", DepthLayer.COGNITIVE, 3),
                DepthPattern(r"もし.*?だとしたら", DepthLayer.COGNITIVE, 3),
            ],
        )

        # 記憶層パターン(時間的・記憶的参照)
        patterns.extend(
            [
                DepthPattern(r"(あの時|昔|思い出)", DepthLayer.MEMORIAL, 2),
                DepthPattern(r"(今は|でも今|なのに)", DepthLayer.MEMORIAL, 1),
                DepthPattern(r"(もし|きっと|いつか)", DepthLayer.MEMORIAL, 1),
                DepthPattern(r".*?年前.*?", DepthLayer.MEMORIAL, 2),
            ],
        )

        # 象徴層パターン(比喩・隠喩・象徴的表現)
        patterns.extend(
            [
                DepthPattern(r"まるで.*?のような", DepthLayer.SYMBOLIC, 2),
                DepthPattern(r"まるで.*?が", DepthLayer.SYMBOLIC, 2),
                DepthPattern(r".*?は.*?だった", DepthLayer.SYMBOLIC, 3),
                DepthPattern(r"値札をつけられた", DepthLayer.SYMBOLIC, 3),
                DepthPattern(r"商品.*?としたら", DepthLayer.SYMBOLIC, 3),
            ],
        )

        return patterns


class ViewpointAwareEvaluator:
    """視点情報を考慮した評価サービス"""

    def adjust_for_viewpoint(
        self, base_score: NarrativeDepthScore, _viewpoint_type: str, complexity_level: str = "低"
    ) -> float:
        """視点情報に基づいてスコアを調整"""
        base_total = base_score.total_score

        # 複雑度による調整
        complexity_adjustments = {
            "低": 1.0,
            "中": 1.1,  # 内省型の場合は少し加点
            "高": 1.2,  # 複雑な視点変更がある場合は大きく加点
        }

        adjustment = complexity_adjustments.get(complexity_level, 1.0)
        adjusted_score = base_total * adjustment

        return min(adjusted_score, 100.0)



#!/usr/bin/env python3
# File: scripts/setup_venv.py
# Purpose: Cross-platform virtual environment setup script for Windows and WSL
# Context: Handles platform-specific venv creation and dependency installation

"""Cross-platform venv setup utility.

This script creates and maintains separate virtual environments for:
- Windows (.venv.win)
- WSL/Linux (.venv.wsl)

Both environments can coexist and are automatically selected based on the
current platform.
"""

import os
import platform
import subprocess
import sys
from pathlib import Path

# Force UTF-8 encoding for Windows console to handle Japanese paths
if platform.system() == "Windows":
    # Set environment variables before any output
    os.environ.setdefault("PYTHONIOENCODING", "utf-8")

    # Reconfigure sys.stdout/stderr to use UTF-8
    if sys.stdout.encoding.lower() != "utf-8":
        import io
        sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding="utf-8", errors="replace")
        sys.stderr = io.TextIOWrapper(sys.stderr.buffer, encoding="utf-8", errors="replace")


class VenvManager:
    """Manage cross-platform virtual environments."""

    def __init__(self, project_root: Path | None = None):
        """Initialize venv manager.

        Args:
            project_root: Project root directory. Defaults to script's parent.
        """
        if project_root is None:
            # Script is in scripts/, project root is parent
            project_root = Path(__file__).parent.parent
        self.project_root = project_root.resolve()
        self.pyproject_toml = self.project_root / "pyproject.toml"

        # Detect platform
        self.is_wsl = self._detect_wsl()
        self.is_windows = platform.system() == "Windows" and not self.is_wsl

        # Select appropriate venv path
        if self.is_windows:
            self.venv_name = ".venv.win"
            self.python_cmd = self._find_windows_python()
            self.venv_python = self.venv_path / "Scripts" / "python.exe"
            self.venv_pip = self.venv_path / "Scripts" / "pip.exe"
        else:  # WSL or Linux
            self.venv_name = ".venv.wsl"
            self.python_cmd = "python3"
            self.venv_python = self.venv_path / "bin" / "python"
            self.venv_pip = self.venv_path / "bin" / "pip"

    @property
    def venv_path(self) -> Path:
        """Get platform-specific venv path."""
        return self.project_root / self.venv_name

    def _detect_wsl(self) -> bool:
        """Detect if running in WSL environment."""
        try:
            with open("/proc/version") as f:
                return "microsoft" in f.read().lower()
        except FileNotFoundError:
            return False

    def _find_windows_python(self) -> str:
        """Find appropriate Python executable on Windows.

        Returns:
            Python command (py, python, python3).

        Raises:
            RuntimeError: If no Python found.
        """
        # Try py launcher first (recommended for Windows)
        for cmd in ["py", "python", "python3"]:
            try:
                result = subprocess.run(
                    [cmd, "--version"],
                    capture_output=True,
                    text=True,
                    check=False,
                )
                if result.returncode == 0:
                    return cmd
            except FileNotFoundError:
                continue

        raise RuntimeError("No Python executable found. Install Python 3.10+")

    def create_venv(self, force: bool = False) -> None:
        """Create platform-specific virtual environment.

        Args:
            force: If True, recreate venv even if it exists.
        """
        if self.venv_path.exists() and not force:
            print(f"[OK] Virtual environment already exists: {self.venv_name}")
            return

        if self.venv_path.exists() and force:
            print(f"[CLEAN] Removing existing venv: {self.venv_name}")
            import shutil
            shutil.rmtree(self.venv_path)

        print(f"[CREATE] Creating virtual environment: {self.venv_name}")
        print(f"         Platform: {'Windows' if self.is_windows else 'WSL/Linux'}")
        print(f"         Python: {self.python_cmd}")

        try:
            subprocess.run(
                [self.python_cmd, "-m", "venv", str(self.venv_path)],
                check=True,
                cwd=self.project_root,
            )
            print("[OK] Virtual environment created successfully")
        except subprocess.CalledProcessError as exc:
            raise RuntimeError(f"Failed to create venv: {exc}") from exc

    def install_dependencies(self, dev: bool = True) -> None:
        """Install dependencies from pyproject.toml.

        Args:
            dev: If True, install development dependencies.
        """
        if not self.venv_path.exists():
            raise RuntimeError(f"Virtual environment not found: {self.venv_name}")

        print("[INSTALL] Installing dependencies...")

        # Upgrade pip first
        print("   Upgrading pip...")
        subprocess.run(
            [str(self.venv_python), "-m", "pip", "install", "--upgrade", "pip"],
            check=True,
            capture_output=True,
        )

        # Install package in editable mode with dependencies
        package_spec = ".[dev]" if dev else "."
        install_cmd = [str(self.venv_pip), "install", "-e", package_spec]

        print(f"   Installing noveler{' [dev]' if dev else ''}...")
        subprocess.run(
            install_cmd,
            check=True,
            cwd=self.project_root,
        )

        print("[OK] Dependencies installed successfully")

    def print_activation_instructions(self) -> None:
        """Print instructions for activating the venv."""
        print("\n" + "=" * 60)
        print("Virtual Environment Setup Complete!")
        print("=" * 60)

        if self.is_windows:
            print("\nTo activate (PowerShell):")
            print(f"  .\\{self.venv_name}\\Scripts\\Activate.ps1")
            print("\nTo activate (Command Prompt):")
            print(f"  {self.venv_name}\\Scripts\\activate.bat")
        else:
            print("\nTo activate:")
            print(f"  source {self.venv_name}/bin/activate")

        print("\nTo deactivate:")
        print("  deactivate")
        print("\n" + "=" * 60)

    def verify_installation(self) -> bool:
        """Verify that the venv is properly configured.

        Returns:
            True if verification passes.
        """
        print("\n[VERIFY] Checking installation...")

        checks = [
            (self.venv_path.exists(), f"Venv directory exists: {self.venv_name}"),
            (self.venv_python.exists(), "Python executable exists"),
            (self.venv_pip.exists(), "Pip executable exists"),
        ]

        # Check if noveler is importable
        try:
            result = subprocess.run(
                [str(self.venv_python), "-c", "import noveler; import sys; print('OK')"],
                capture_output=True,
                text=True,
                check=True,
            )
            if result.stdout.strip() == "OK":
                checks.append((True, "Noveler package installed and importable"))
            else:
                checks.append((False, f"Unexpected output: {result.stdout.strip()}"))
        except subprocess.CalledProcessError as exc:
            checks.append((False, f"Noveler package NOT importable: {exc.stderr}"))

        all_passed = all(passed for passed, _ in checks)

        for passed, message in checks:
            print(f"  {'[OK]' if passed else '[FAIL]'} {message}")

        if all_passed:
            print("\n[OK] All checks passed!")
        else:
            print("\n[FAIL] Some checks failed. Please review the output above.")

        return all_passed


def main() -> int:
    """Main entry point.

    Returns:
        Exit code (0 for success).
    """
    import argparse

    parser = argparse.ArgumentParser(
        description="Setup cross-platform virtual environment for noveler project"
    )
    parser.add_argument(
        "--force",
        action="store_true",
        help="Force recreate venv even if it exists",
    )
    parser.add_argument(
        "--no-dev",
        action="store_true",
        help="Skip development dependencies",
    )
    parser.add_argument(
        "--verify-only",
        action="store_true",
        help="Only verify existing venv, don't create or install",
    )
    args = parser.parse_args()

    try:
        manager = VenvManager()

        print("Noveler Virtual Environment Setup")
        print(f"  Project: {manager.project_root}")
        print(f"  Platform: {'Windows' if manager.is_windows else 'WSL/Linux'}")
        print(f"  Target: {manager.venv_name}\n")

        if args.verify_only:
            if not manager.venv_path.exists():
                print(f"[FAIL] Virtual environment not found: {manager.venv_name}")
                return 1
            return 0 if manager.verify_installation() else 1

        # Create venv
        manager.create_venv(force=args.force)

        # Install dependencies
        manager.install_dependencies(dev=not args.no_dev)

        # Verify
        if not manager.verify_installation():
            return 1

        # Print activation instructions
        manager.print_activation_instructions()

        return 0

    except Exception as exc:
        print(f"\n[ERROR] {exc}", file=sys.stderr)
        return 1


if __name__ == "__main__":
    sys.exit(main())



# File: tests/unit/infrastructure/adapters/test_path_service_report_filename.py
# Purpose: Integration tests for report filename generation via PathService
# Context: Validates that PathServiceAdapter correctly delegates to ReportFileNamingService

from pathlib import Path

import pytest

from noveler.infrastructure.adapters.path_service_adapter import PathServiceAdapter


class TestPathServiceReportFilenameGeneration:
    """PathServiceAdapter経由のレポートファイル名生成テスト"""

    @pytest.fixture
    def path_service(self, tmp_path: Path) -> PathServiceAdapter:
        """PathServiceAdapterインスタンス"""
        return PathServiceAdapter(project_root=tmp_path)

    def test_basic_report_filename_generation(
        self, path_service: PathServiceAdapter
    ) -> None:
        """基本的なレポートファイル名生成"""
        filename = path_service.generate_report_filename(
            episode_number=1, report_type="A41", extension="md"
        )
        assert filename == "episode_001_A41.md"

    def test_report_filename_with_json_extension(
        self, path_service: PathServiceAdapter
    ) -> None:
        """JSON拡張子でのレポートファイル名生成"""
        filename = path_service.generate_report_filename(
            episode_number=42, report_type="quality", extension="json"
        )
        assert filename == "episode_042_quality.json"

    def test_report_filename_with_yaml_extension(
        self, path_service: PathServiceAdapter
    ) -> None:
        """YAML拡張子でのレポートファイル名生成"""
        filename = path_service.generate_report_filename(
            episode_number=999, report_type="backup", extension="yaml"
        )
        assert filename == "episode_999_backup.yaml"

    def test_report_filename_with_version(
        self, path_service: PathServiceAdapter
    ) -> None:
        """バージョン付きレポートファイル名生成"""
        filename = path_service.generate_report_filename(
            episode_number=1, report_type="A41", extension="md", version="v2"
        )
        assert filename == "episode_001_A41_v2.md"

    def test_report_filename_with_timestamp(
        self, path_service: PathServiceAdapter
    ) -> None:
        """タイムスタンプ付きレポートファイル名生成"""
        filename = path_service.generate_report_filename(
            episode_number=1,
            report_type="backup",
            extension="yaml",
            include_timestamp=True,
        )
        # タイムスタンプの正確な値は予測不可能
        assert filename.startswith("episode_001_backup_")
        assert filename.endswith(".yaml")

    def test_report_filename_with_timestamp_and_version(
        self, path_service: PathServiceAdapter
    ) -> None:
        """タイムスタンプとバージョン付きレポートファイル名生成"""
        filename = path_service.generate_report_filename(
            episode_number=5,
            report_type="quality",
            extension="json",
            include_timestamp=True,
            version="v3",
        )
        assert filename.startswith("episode_005_quality_")
        assert "_v3.json" in filename

    def test_report_filename_sorting_consistency(
        self, path_service: PathServiceAdapter
    ) -> None:
        """複数エピソードのレポートファイル名がソート可能"""
        filenames = [
            path_service.generate_report_filename(i, "A41", "md")
            for i in [1, 2, 10, 100, 999]
        ]

        # 自然順序でソート
        assert filenames == sorted(filenames)
        assert filenames[0] == "episode_001_A41.md"
        assert filenames[1] == "episode_002_A41.md"
        assert filenames[2] == "episode_010_A41.md"
        assert filenames[3] == "episode_100_A41.md"
        assert filenames[4] == "episode_999_A41.md"

    def test_report_filename_invalid_episode_number_too_low(
        self, path_service: PathServiceAdapter
    ) -> None:
        """エピソード番号が範囲外（下限）でエラー"""
        with pytest.raises(ValueError, match="must be between 1 and 999"):
            path_service.generate_report_filename(
                episode_number=0, report_type="A41", extension="md"
            )

    def test_report_filename_invalid_episode_number_too_high(
        self, path_service: PathServiceAdapter
    ) -> None:
        """エピソード番号が範囲外（上限）でエラー"""
        with pytest.raises(ValueError, match="must be between 1 and 999"):
            path_service.generate_report_filename(
                episode_number=1000, report_type="A41", extension="md"
            )

    def test_full_path_generation_with_reports_dir(
        self, path_service: PathServiceAdapter
    ) -> None:
        """reportsディレクトリと組み合わせた完全パス生成"""
        reports_dir = path_service.get_reports_dir()
        quality_dir = reports_dir / "quality"
        quality_dir.mkdir(parents=True, exist_ok=True)

        filename = path_service.generate_report_filename(
            episode_number=1, report_type="A41", extension="md"
        )

        full_path = quality_dir / filename
        # パスの末尾部分を検証（プラットフォーム独立）
        assert full_path.name == "episode_001_A41.md"
        assert full_path.parent.name == "quality"

    def test_different_report_types(
        self, path_service: PathServiceAdapter
    ) -> None:
        """異なるレポートタイプでのファイル名生成"""
        report_types = ["A41", "quality", "backup", "summary", "analysis"]

        for report_type in report_types:
            filename = path_service.generate_report_filename(
                episode_number=1, report_type=report_type, extension="md"
            )
            assert filename == f"episode_001_{report_type}.md"



"""章番号値オブジェクトのテスト

SPEC-PLOT-001: Claude Code連携プロット生成システム
"""

import pytest

from noveler.domain.value_objects.chapter_number import ChapterNumber, ChapterNumberError

pytestmark = pytest.mark.vo_smoke



class TestChapterNumber:
    """章番号値オブジェクトのテストクラス"""

    @pytest.mark.spec("SPEC-PLOT-001")
    def test_chapter_number_creation_success(self) -> None:
        """章番号作成成功テスト"""
        # Given & When: 有効な章番号で作成
        chapter_number = ChapterNumber(1)

        # Then: 正しく作成される
        assert chapter_number.value == 1
        assert str(chapter_number) == "chapter01"
        assert repr(chapter_number) == "ChapterNumber(1)"

    @pytest.mark.spec("SPEC-PLOT-001")
    def test_chapter_number_creation_with_large_number(self) -> None:
        """大きな章番号作成テスト"""
        # Given & When: 大きな章番号で作成
        chapter_number = ChapterNumber(99)

        # Then: 正しく作成される
        assert chapter_number.value == 99
        assert str(chapter_number) == "chapter99"

    @pytest.mark.spec("SPEC-PLOT-001")
    def test_chapter_number_creation_failure_zero(self) -> None:
        """章番号作成失敗テスト(0)"""
        # Given & When & Then: 0で作成するとエラー
        with pytest.raises(ChapterNumberError, match="章番号は1以上の整数である必要があります"):
            ChapterNumber(0)

    @pytest.mark.spec("SPEC-PLOT-001")
    def test_chapter_number_creation_failure_negative(self) -> None:
        """章番号作成失敗テスト(負数)"""
        # Given & When & Then: 負数で作成するとエラー
        with pytest.raises(ChapterNumberError, match="章番号は1以上の整数である必要があります"):
            ChapterNumber(-1)

    @pytest.mark.spec("SPEC-PLOT-001")
    def test_chapter_number_equality(self) -> None:
        """章番号等価性テスト"""
        # Given: 同じ値の章番号
        chapter1 = ChapterNumber(1)
        chapter2 = ChapterNumber(1)
        chapter3 = ChapterNumber(2)

        # When & Then: 等価性チェック
        assert chapter1 == chapter2
        assert chapter1 != chapter3
        assert hash(chapter1) == hash(chapter2)
        assert hash(chapter1) != hash(chapter3)

    @pytest.mark.spec("SPEC-PLOT-001")
    def test_chapter_number_comparison(self) -> None:
        """章番号比較テスト"""
        # Given: 異なる章番号
        chapter1 = ChapterNumber(1)
        chapter2 = ChapterNumber(2)
        chapter3 = ChapterNumber(3)

        # When & Then: 比較演算子テスト
        assert chapter1 < chapter2
        assert chapter2 < chapter3
        assert chapter1 <= chapter2
        assert chapter2 <= chapter2
        assert chapter3 > chapter2
        assert chapter3 >= chapter2
        assert chapter2 >= chapter2

    @pytest.mark.spec("SPEC-PLOT-001")
    def test_episode_number_to_chapter_number(self) -> None:
        """エピソード番号から章番号変換テスト"""
        # Given & When & Then: エピソード番号から章番号を計算
        # 仮定: 1章あたり10エピソード
        assert ChapterNumber.from_episode_number(1) == ChapterNumber(1)
        assert ChapterNumber.from_episode_number(5) == ChapterNumber(1)
        assert ChapterNumber.from_episode_number(10) == ChapterNumber(1)
        assert ChapterNumber.from_episode_number(11) == ChapterNumber(2)
        assert ChapterNumber.from_episode_number(15) == ChapterNumber(2)
        assert ChapterNumber.from_episode_number(20) == ChapterNumber(2)
        assert ChapterNumber.from_episode_number(21) == ChapterNumber(3)

    @pytest.mark.spec("SPEC-PLOT-001")
    def test_episode_number_to_chapter_number_custom_episodes_per_chapter(self) -> None:
        """カスタムエピソード数での章番号変換テスト"""
        # Given & When & Then: カスタムエピソード数での変換
        # 1章あたり5エピソード
        assert ChapterNumber.from_episode_number(1, episodes_per_chapter=5) == ChapterNumber(1)
        assert ChapterNumber.from_episode_number(5, episodes_per_chapter=5) == ChapterNumber(1)
        assert ChapterNumber.from_episode_number(6, episodes_per_chapter=5) == ChapterNumber(2)
        assert ChapterNumber.from_episode_number(10, episodes_per_chapter=5) == ChapterNumber(2)
        assert ChapterNumber.from_episode_number(11, episodes_per_chapter=5) == ChapterNumber(3)

    @pytest.mark.spec("SPEC-PLOT-001")
    def test_episode_number_to_chapter_number_failure(self) -> None:
        """エピソード番号から章番号変換失敗テスト"""
        # Given & When & Then: 無効なエピソード番号でエラー
        with pytest.raises(ChapterNumberError, match="エピソード番号は1以上の整数である必要があります"):
            ChapterNumber.from_episode_number(0)

        with pytest.raises(ChapterNumberError, match="エピソード番号は1以上の整数である必要があります"):
            ChapterNumber.from_episode_number(-1)



"""ローカルファイルシステムリポジトリ実装

ファイルI/Oとハッシュ計算の実装をインフラ層に配置
"""

import hashlib
import time
from datetime import datetime, timezone
from pathlib import Path
from typing import Any

import yaml

from noveler.domain.repositories.file_system_repository import FileSystemRepository


class LocalFileSystemRepository(FileSystemRepository):
    """ローカルファイルシステムベースのリポジトリ"""

    def exists(self, path: str | Path) -> bool:
        """パスの存在を確認する

        Args:
            path: 確認対象のパス(文字列)

        Returns:
            パスが存在する場合True
        """
        return Path(path).exists()

    def is_directory(self, path: str | Path) -> bool:
        """パスがディレクトリかどうか確認する

        Args:
            path: 確認対象のパス(文字列)

        Returns:
            パスがディレクトリの場合True
        """
        return Path(path).is_dir()

    def list_files(self, directory: str, extensions: set[str]) -> list[str]:
        """指定拡張子のファイルをリストアップする

        Args:
            directory: 検索対象ディレクトリ
            extensions: 対象拡張子のセット(例: {'.yaml', '.yml'})

        Returns:
            マッチしたファイルのリスト
        """
        directory_path = Path(directory)
        if not directory_path.exists() or not directory_path.is_dir():
            return []

        return [
            str(file_path)
            for file_path in directory_path.iterdir()
            if file_path.is_file() and file_path.suffix.lower() in extensions
        ]

    def get_file_info(self, file_path: str | Path) -> dict[str, str | int | float] | None:
        """ファイルの状態情報を取得する

        Args:
            file_path: ファイルパス(文字列)

        Returns:
            ファイル状態情報の辞書(mtime, size, hash等)
            ファイルが存在しない場合はNone
        """
        try:
            path_obj = Path(file_path)
            if not path_obj.exists():
                return None

            stat = path_obj.stat()

            # ファイルハッシュ計算
            file_hash = self.calculate_hash(file_path)

            return {
                "mtime": stat.st_mtime,
                "size": stat.st_size,
                "hash": file_hash,
                "last_checked": time.time(),
                "path": str(file_path),
            }
        except OSError:
            # ファイルアクセスエラーは通常の状況として扱う
            return None

    def calculate_hash(self, file_path: str | Path) -> str | None:
        """ファイルのハッシュ値を計算する

        Args:
            file_path: ファイルパス(文字列)

        Returns:
            ハッシュ値(MD5)
            ファイルが読み込めない場合はNone
        """
        try:
            with Path(file_path).open("rb") as f:
                return hashlib.md5(f.read()).hexdigest()
        except OSError:
            return None

    def get_modification_time(self, file_path: str | Path) -> datetime | None:
        """ファイルの最終更新時刻を取得

        Args:
            file_path: ファイルパス(文字列)

        Returns:
            最終更新時刻(取得失敗時None)
        """
        try:
            path_obj = Path(file_path)
            if path_obj.exists():
                return datetime.fromtimestamp(path_obj.stat().st_mtime, tz=timezone.utc)
            return None
        except OSError:
            return None

    def read_yaml(self, file_path: str | Path) -> dict[str, Any] | None:
        """YAMLファイルを読み込み

        Args:
            file_path: YAMLファイルパス(文字列)

        Returns:
            パースされたデータ(読み込み失敗時None)
        """
        try:
            with Path(file_path).open(encoding="utf-8") as f:
                return yaml.safe_load(f)
        except (OSError, yaml.YAMLError):
            return None

    def write_yaml(self, file_path: str | Path, data: dict[str, Any]) -> bool:
        """YAMLファイルに書き込み

        Args:
            file_path: YAMLファイルパス(文字列)
            data: 書き込むデータ

        Returns:
            成功時True
        """
        try:
            path_obj = Path(file_path)
            path_obj.parent.mkdir(parents=True, exist_ok=True)
            with Path(path_obj).open("w", encoding="utf-8") as f:
                yaml.dump(data, f, allow_unicode=True, default_flow_style=False)
            return True
        except (OSError, yaml.YAMLError):
            return False
